//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package fake

import (
	"context"
	"errors"
	"fmt"
	"net/http"
	"net/url"
	"regexp"
	"strconv"

	azfake "github.com/Azure/azure-sdk-for-go/sdk/azcore/fake"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/fake/server"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/to"
	"github.com/Azure/azure-sdk-for-go/sdk/resourcemanager/billing/armbilling"
)

// SavingsPlansServer is a fake server for instances of the armbilling.SavingsPlansClient type.
type SavingsPlansServer struct {
	// GetByBillingAccount is the fake for method SavingsPlansClient.GetByBillingAccount
	// HTTP status codes to indicate success: http.StatusOK
	GetByBillingAccount func(ctx context.Context, billingAccountName string, savingsPlanOrderID string, savingsPlanID string, options *armbilling.SavingsPlansClientGetByBillingAccountOptions) (resp azfake.Responder[armbilling.SavingsPlansClientGetByBillingAccountResponse], errResp azfake.ErrorResponder)

	// NewListByBillingAccountPager is the fake for method SavingsPlansClient.NewListByBillingAccountPager
	// HTTP status codes to indicate success: http.StatusOK
	NewListByBillingAccountPager func(billingAccountName string, options *armbilling.SavingsPlansClientListByBillingAccountOptions) (resp azfake.PagerResponder[armbilling.SavingsPlansClientListByBillingAccountResponse])

	// NewListBySavingsPlanOrderPager is the fake for method SavingsPlansClient.NewListBySavingsPlanOrderPager
	// HTTP status codes to indicate success: http.StatusOK
	NewListBySavingsPlanOrderPager func(billingAccountName string, savingsPlanOrderID string, options *armbilling.SavingsPlansClientListBySavingsPlanOrderOptions) (resp azfake.PagerResponder[armbilling.SavingsPlansClientListBySavingsPlanOrderResponse])

	// BeginUpdateByBillingAccount is the fake for method SavingsPlansClient.BeginUpdateByBillingAccount
	// HTTP status codes to indicate success: http.StatusOK, http.StatusAccepted
	BeginUpdateByBillingAccount func(ctx context.Context, billingAccountName string, savingsPlanOrderID string, savingsPlanID string, body armbilling.SavingsPlanUpdateRequest, options *armbilling.SavingsPlansClientBeginUpdateByBillingAccountOptions) (resp azfake.PollerResponder[armbilling.SavingsPlansClientUpdateByBillingAccountResponse], errResp azfake.ErrorResponder)

	// ValidateUpdateByBillingAccount is the fake for method SavingsPlansClient.ValidateUpdateByBillingAccount
	// HTTP status codes to indicate success: http.StatusOK
	ValidateUpdateByBillingAccount func(ctx context.Context, billingAccountName string, savingsPlanOrderID string, savingsPlanID string, body armbilling.SavingsPlanUpdateValidateRequest, options *armbilling.SavingsPlansClientValidateUpdateByBillingAccountOptions) (resp azfake.Responder[armbilling.SavingsPlansClientValidateUpdateByBillingAccountResponse], errResp azfake.ErrorResponder)
}

// NewSavingsPlansServerTransport creates a new instance of SavingsPlansServerTransport with the provided implementation.
// The returned SavingsPlansServerTransport instance is connected to an instance of armbilling.SavingsPlansClient via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewSavingsPlansServerTransport(srv *SavingsPlansServer) *SavingsPlansServerTransport {
	return &SavingsPlansServerTransport{
		srv:                            srv,
		newListByBillingAccountPager:   newTracker[azfake.PagerResponder[armbilling.SavingsPlansClientListByBillingAccountResponse]](),
		newListBySavingsPlanOrderPager: newTracker[azfake.PagerResponder[armbilling.SavingsPlansClientListBySavingsPlanOrderResponse]](),
		beginUpdateByBillingAccount:    newTracker[azfake.PollerResponder[armbilling.SavingsPlansClientUpdateByBillingAccountResponse]](),
	}
}

// SavingsPlansServerTransport connects instances of armbilling.SavingsPlansClient to instances of SavingsPlansServer.
// Don't use this type directly, use NewSavingsPlansServerTransport instead.
type SavingsPlansServerTransport struct {
	srv                            *SavingsPlansServer
	newListByBillingAccountPager   *tracker[azfake.PagerResponder[armbilling.SavingsPlansClientListByBillingAccountResponse]]
	newListBySavingsPlanOrderPager *tracker[azfake.PagerResponder[armbilling.SavingsPlansClientListBySavingsPlanOrderResponse]]
	beginUpdateByBillingAccount    *tracker[azfake.PollerResponder[armbilling.SavingsPlansClientUpdateByBillingAccountResponse]]
}

// Do implements the policy.Transporter interface for SavingsPlansServerTransport.
func (s *SavingsPlansServerTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	var resp *http.Response
	var err error

	switch method {
	case "SavingsPlansClient.GetByBillingAccount":
		resp, err = s.dispatchGetByBillingAccount(req)
	case "SavingsPlansClient.NewListByBillingAccountPager":
		resp, err = s.dispatchNewListByBillingAccountPager(req)
	case "SavingsPlansClient.NewListBySavingsPlanOrderPager":
		resp, err = s.dispatchNewListBySavingsPlanOrderPager(req)
	case "SavingsPlansClient.BeginUpdateByBillingAccount":
		resp, err = s.dispatchBeginUpdateByBillingAccount(req)
	case "SavingsPlansClient.ValidateUpdateByBillingAccount":
		resp, err = s.dispatchValidateUpdateByBillingAccount(req)
	default:
		err = fmt.Errorf("unhandled API %s", method)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func (s *SavingsPlansServerTransport) dispatchGetByBillingAccount(req *http.Request) (*http.Response, error) {
	if s.srv.GetByBillingAccount == nil {
		return nil, &nonRetriableError{errors.New("fake for method GetByBillingAccount not implemented")}
	}
	const regexStr = `/providers/Microsoft\.Billing/billingAccounts/(?P<billingAccountName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/savingsPlanOrders/(?P<savingsPlanOrderId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/savingsPlans/(?P<savingsPlanId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)`
	regex := regexp.MustCompile(regexStr)
	matches := regex.FindStringSubmatch(req.URL.EscapedPath())
	if matches == nil || len(matches) < 3 {
		return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
	}
	qp := req.URL.Query()
	billingAccountNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("billingAccountName")])
	if err != nil {
		return nil, err
	}
	savingsPlanOrderIDParam, err := url.PathUnescape(matches[regex.SubexpIndex("savingsPlanOrderId")])
	if err != nil {
		return nil, err
	}
	savingsPlanIDParam, err := url.PathUnescape(matches[regex.SubexpIndex("savingsPlanId")])
	if err != nil {
		return nil, err
	}
	expandUnescaped, err := url.QueryUnescape(qp.Get("expand"))
	if err != nil {
		return nil, err
	}
	expandParam := getOptional(expandUnescaped)
	var options *armbilling.SavingsPlansClientGetByBillingAccountOptions
	if expandParam != nil {
		options = &armbilling.SavingsPlansClientGetByBillingAccountOptions{
			Expand: expandParam,
		}
	}
	respr, errRespr := s.srv.GetByBillingAccount(req.Context(), billingAccountNameParam, savingsPlanOrderIDParam, savingsPlanIDParam, options)
	if respErr := server.GetError(errRespr, req); respErr != nil {
		return nil, respErr
	}
	respContent := server.GetResponseContent(respr)
	if !contains([]int{http.StatusOK}, respContent.HTTPStatus) {
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", respContent.HTTPStatus)}
	}
	resp, err := server.MarshalResponseAsJSON(respContent, server.GetResponse(respr).SavingsPlanModel, req)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func (s *SavingsPlansServerTransport) dispatchNewListByBillingAccountPager(req *http.Request) (*http.Response, error) {
	if s.srv.NewListByBillingAccountPager == nil {
		return nil, &nonRetriableError{errors.New("fake for method NewListByBillingAccountPager not implemented")}
	}
	newListByBillingAccountPager := s.newListByBillingAccountPager.get(req)
	if newListByBillingAccountPager == nil {
		const regexStr = `/providers/Microsoft\.Billing/billingAccounts/(?P<billingAccountName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/savingsPlans`
		regex := regexp.MustCompile(regexStr)
		matches := regex.FindStringSubmatch(req.URL.EscapedPath())
		if matches == nil || len(matches) < 1 {
			return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
		}
		qp := req.URL.Query()
		billingAccountNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("billingAccountName")])
		if err != nil {
			return nil, err
		}
		filterUnescaped, err := url.QueryUnescape(qp.Get("filter"))
		if err != nil {
			return nil, err
		}
		filterParam := getOptional(filterUnescaped)
		orderByUnescaped, err := url.QueryUnescape(qp.Get("orderBy"))
		if err != nil {
			return nil, err
		}
		orderByParam := getOptional(orderByUnescaped)
		skiptokenUnescaped, err := url.QueryUnescape(qp.Get("skiptoken"))
		if err != nil {
			return nil, err
		}
		skiptokenParam, err := parseOptional(skiptokenUnescaped, func(v string) (float32, error) {
			p, parseErr := strconv.ParseFloat(v, 32)
			if parseErr != nil {
				return 0, parseErr
			}
			return float32(p), nil
		})
		if err != nil {
			return nil, err
		}
		takeUnescaped, err := url.QueryUnescape(qp.Get("take"))
		if err != nil {
			return nil, err
		}
		takeParam, err := parseOptional(takeUnescaped, func(v string) (float32, error) {
			p, parseErr := strconv.ParseFloat(v, 32)
			if parseErr != nil {
				return 0, parseErr
			}
			return float32(p), nil
		})
		if err != nil {
			return nil, err
		}
		selectedStateUnescaped, err := url.QueryUnescape(qp.Get("selectedState"))
		if err != nil {
			return nil, err
		}
		selectedStateParam := getOptional(selectedStateUnescaped)
		refreshSummaryUnescaped, err := url.QueryUnescape(qp.Get("refreshSummary"))
		if err != nil {
			return nil, err
		}
		refreshSummaryParam := getOptional(refreshSummaryUnescaped)
		var options *armbilling.SavingsPlansClientListByBillingAccountOptions
		if filterParam != nil || orderByParam != nil || skiptokenParam != nil || takeParam != nil || selectedStateParam != nil || refreshSummaryParam != nil {
			options = &armbilling.SavingsPlansClientListByBillingAccountOptions{
				Filter:         filterParam,
				OrderBy:        orderByParam,
				Skiptoken:      skiptokenParam,
				Take:           takeParam,
				SelectedState:  selectedStateParam,
				RefreshSummary: refreshSummaryParam,
			}
		}
		resp := s.srv.NewListByBillingAccountPager(billingAccountNameParam, options)
		newListByBillingAccountPager = &resp
		s.newListByBillingAccountPager.add(req, newListByBillingAccountPager)
		server.PagerResponderInjectNextLinks(newListByBillingAccountPager, req, func(page *armbilling.SavingsPlansClientListByBillingAccountResponse, createLink func() string) {
			page.NextLink = to.Ptr(createLink())
		})
	}
	resp, err := server.PagerResponderNext(newListByBillingAccountPager, req)
	if err != nil {
		return nil, err
	}
	if !contains([]int{http.StatusOK}, resp.StatusCode) {
		s.newListByBillingAccountPager.remove(req)
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", resp.StatusCode)}
	}
	if !server.PagerResponderMore(newListByBillingAccountPager) {
		s.newListByBillingAccountPager.remove(req)
	}
	return resp, nil
}

func (s *SavingsPlansServerTransport) dispatchNewListBySavingsPlanOrderPager(req *http.Request) (*http.Response, error) {
	if s.srv.NewListBySavingsPlanOrderPager == nil {
		return nil, &nonRetriableError{errors.New("fake for method NewListBySavingsPlanOrderPager not implemented")}
	}
	newListBySavingsPlanOrderPager := s.newListBySavingsPlanOrderPager.get(req)
	if newListBySavingsPlanOrderPager == nil {
		const regexStr = `/providers/Microsoft\.Billing/billingAccounts/(?P<billingAccountName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/savingsPlanOrders/(?P<savingsPlanOrderId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/savingsPlans`
		regex := regexp.MustCompile(regexStr)
		matches := regex.FindStringSubmatch(req.URL.EscapedPath())
		if matches == nil || len(matches) < 2 {
			return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
		}
		billingAccountNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("billingAccountName")])
		if err != nil {
			return nil, err
		}
		savingsPlanOrderIDParam, err := url.PathUnescape(matches[regex.SubexpIndex("savingsPlanOrderId")])
		if err != nil {
			return nil, err
		}
		resp := s.srv.NewListBySavingsPlanOrderPager(billingAccountNameParam, savingsPlanOrderIDParam, nil)
		newListBySavingsPlanOrderPager = &resp
		s.newListBySavingsPlanOrderPager.add(req, newListBySavingsPlanOrderPager)
		server.PagerResponderInjectNextLinks(newListBySavingsPlanOrderPager, req, func(page *armbilling.SavingsPlansClientListBySavingsPlanOrderResponse, createLink func() string) {
			page.NextLink = to.Ptr(createLink())
		})
	}
	resp, err := server.PagerResponderNext(newListBySavingsPlanOrderPager, req)
	if err != nil {
		return nil, err
	}
	if !contains([]int{http.StatusOK}, resp.StatusCode) {
		s.newListBySavingsPlanOrderPager.remove(req)
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", resp.StatusCode)}
	}
	if !server.PagerResponderMore(newListBySavingsPlanOrderPager) {
		s.newListBySavingsPlanOrderPager.remove(req)
	}
	return resp, nil
}

func (s *SavingsPlansServerTransport) dispatchBeginUpdateByBillingAccount(req *http.Request) (*http.Response, error) {
	if s.srv.BeginUpdateByBillingAccount == nil {
		return nil, &nonRetriableError{errors.New("fake for method BeginUpdateByBillingAccount not implemented")}
	}
	beginUpdateByBillingAccount := s.beginUpdateByBillingAccount.get(req)
	if beginUpdateByBillingAccount == nil {
		const regexStr = `/providers/Microsoft\.Billing/billingAccounts/(?P<billingAccountName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/savingsPlanOrders/(?P<savingsPlanOrderId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/savingsPlans/(?P<savingsPlanId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)`
		regex := regexp.MustCompile(regexStr)
		matches := regex.FindStringSubmatch(req.URL.EscapedPath())
		if matches == nil || len(matches) < 3 {
			return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
		}
		body, err := server.UnmarshalRequestAsJSON[armbilling.SavingsPlanUpdateRequest](req)
		if err != nil {
			return nil, err
		}
		billingAccountNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("billingAccountName")])
		if err != nil {
			return nil, err
		}
		savingsPlanOrderIDParam, err := url.PathUnescape(matches[regex.SubexpIndex("savingsPlanOrderId")])
		if err != nil {
			return nil, err
		}
		savingsPlanIDParam, err := url.PathUnescape(matches[regex.SubexpIndex("savingsPlanId")])
		if err != nil {
			return nil, err
		}
		respr, errRespr := s.srv.BeginUpdateByBillingAccount(req.Context(), billingAccountNameParam, savingsPlanOrderIDParam, savingsPlanIDParam, body, nil)
		if respErr := server.GetError(errRespr, req); respErr != nil {
			return nil, respErr
		}
		beginUpdateByBillingAccount = &respr
		s.beginUpdateByBillingAccount.add(req, beginUpdateByBillingAccount)
	}

	resp, err := server.PollerResponderNext(beginUpdateByBillingAccount, req)
	if err != nil {
		return nil, err
	}

	if !contains([]int{http.StatusOK, http.StatusAccepted}, resp.StatusCode) {
		s.beginUpdateByBillingAccount.remove(req)
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK, http.StatusAccepted", resp.StatusCode)}
	}
	if !server.PollerResponderMore(beginUpdateByBillingAccount) {
		s.beginUpdateByBillingAccount.remove(req)
	}

	return resp, nil
}

func (s *SavingsPlansServerTransport) dispatchValidateUpdateByBillingAccount(req *http.Request) (*http.Response, error) {
	if s.srv.ValidateUpdateByBillingAccount == nil {
		return nil, &nonRetriableError{errors.New("fake for method ValidateUpdateByBillingAccount not implemented")}
	}
	const regexStr = `/providers/Microsoft\.Billing/billingAccounts/(?P<billingAccountName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/savingsPlanOrders/(?P<savingsPlanOrderId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/savingsPlans/(?P<savingsPlanId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/validate`
	regex := regexp.MustCompile(regexStr)
	matches := regex.FindStringSubmatch(req.URL.EscapedPath())
	if matches == nil || len(matches) < 3 {
		return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
	}
	body, err := server.UnmarshalRequestAsJSON[armbilling.SavingsPlanUpdateValidateRequest](req)
	if err != nil {
		return nil, err
	}
	billingAccountNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("billingAccountName")])
	if err != nil {
		return nil, err
	}
	savingsPlanOrderIDParam, err := url.PathUnescape(matches[regex.SubexpIndex("savingsPlanOrderId")])
	if err != nil {
		return nil, err
	}
	savingsPlanIDParam, err := url.PathUnescape(matches[regex.SubexpIndex("savingsPlanId")])
	if err != nil {
		return nil, err
	}
	respr, errRespr := s.srv.ValidateUpdateByBillingAccount(req.Context(), billingAccountNameParam, savingsPlanOrderIDParam, savingsPlanIDParam, body, nil)
	if respErr := server.GetError(errRespr, req); respErr != nil {
		return nil, respErr
	}
	respContent := server.GetResponseContent(respr)
	if !contains([]int{http.StatusOK}, respContent.HTTPStatus) {
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", respContent.HTTPStatus)}
	}
	resp, err := server.MarshalResponseAsJSON(respContent, server.GetResponse(respr).SavingsPlanValidateResponse, req)
	if err != nil {
		return nil, err
	}
	return resp, nil
}
