//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package fake

import (
	"errors"
	"fmt"
	azfake "github.com/Azure/azure-sdk-for-go/sdk/azcore/fake"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/fake/server"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/to"
	"github.com/Azure/azure-sdk-for-go/sdk/resourcemanager/cognitiveservices/armcognitiveservices"
	"net/http"
	"net/url"
	"regexp"
)

// LocationBasedModelCapacitiesServer is a fake server for instances of the armcognitiveservices.LocationBasedModelCapacitiesClient type.
type LocationBasedModelCapacitiesServer struct {
	// NewListPager is the fake for method LocationBasedModelCapacitiesClient.NewListPager
	// HTTP status codes to indicate success: http.StatusOK
	NewListPager func(location string, modelFormat string, modelName string, modelVersion string, options *armcognitiveservices.LocationBasedModelCapacitiesClientListOptions) (resp azfake.PagerResponder[armcognitiveservices.LocationBasedModelCapacitiesClientListResponse])
}

// NewLocationBasedModelCapacitiesServerTransport creates a new instance of LocationBasedModelCapacitiesServerTransport with the provided implementation.
// The returned LocationBasedModelCapacitiesServerTransport instance is connected to an instance of armcognitiveservices.LocationBasedModelCapacitiesClient via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewLocationBasedModelCapacitiesServerTransport(srv *LocationBasedModelCapacitiesServer) *LocationBasedModelCapacitiesServerTransport {
	return &LocationBasedModelCapacitiesServerTransport{
		srv:          srv,
		newListPager: newTracker[azfake.PagerResponder[armcognitiveservices.LocationBasedModelCapacitiesClientListResponse]](),
	}
}

// LocationBasedModelCapacitiesServerTransport connects instances of armcognitiveservices.LocationBasedModelCapacitiesClient to instances of LocationBasedModelCapacitiesServer.
// Don't use this type directly, use NewLocationBasedModelCapacitiesServerTransport instead.
type LocationBasedModelCapacitiesServerTransport struct {
	srv          *LocationBasedModelCapacitiesServer
	newListPager *tracker[azfake.PagerResponder[armcognitiveservices.LocationBasedModelCapacitiesClientListResponse]]
}

// Do implements the policy.Transporter interface for LocationBasedModelCapacitiesServerTransport.
func (l *LocationBasedModelCapacitiesServerTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	var resp *http.Response
	var err error

	switch method {
	case "LocationBasedModelCapacitiesClient.NewListPager":
		resp, err = l.dispatchNewListPager(req)
	default:
		err = fmt.Errorf("unhandled API %s", method)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func (l *LocationBasedModelCapacitiesServerTransport) dispatchNewListPager(req *http.Request) (*http.Response, error) {
	if l.srv.NewListPager == nil {
		return nil, &nonRetriableError{errors.New("fake for method NewListPager not implemented")}
	}
	newListPager := l.newListPager.get(req)
	if newListPager == nil {
		const regexStr = `/subscriptions/(?P<subscriptionId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Microsoft\.CognitiveServices/locations/(?P<location>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/modelCapacities`
		regex := regexp.MustCompile(regexStr)
		matches := regex.FindStringSubmatch(req.URL.EscapedPath())
		if matches == nil || len(matches) < 2 {
			return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
		}
		qp := req.URL.Query()
		locationParam, err := url.PathUnescape(matches[regex.SubexpIndex("location")])
		if err != nil {
			return nil, err
		}
		modelFormatParam, err := url.QueryUnescape(qp.Get("modelFormat"))
		if err != nil {
			return nil, err
		}
		modelNameParam, err := url.QueryUnescape(qp.Get("modelName"))
		if err != nil {
			return nil, err
		}
		modelVersionParam, err := url.QueryUnescape(qp.Get("modelVersion"))
		if err != nil {
			return nil, err
		}
		resp := l.srv.NewListPager(locationParam, modelFormatParam, modelNameParam, modelVersionParam, nil)
		newListPager = &resp
		l.newListPager.add(req, newListPager)
		server.PagerResponderInjectNextLinks(newListPager, req, func(page *armcognitiveservices.LocationBasedModelCapacitiesClientListResponse, createLink func() string) {
			page.NextLink = to.Ptr(createLink())
		})
	}
	resp, err := server.PagerResponderNext(newListPager, req)
	if err != nil {
		return nil, err
	}
	if !contains([]int{http.StatusOK}, resp.StatusCode) {
		l.newListPager.remove(req)
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", resp.StatusCode)}
	}
	if !server.PagerResponderMore(newListPager) {
		l.newListPager.remove(req)
	}
	return resp, nil
}
