//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package fake

import (
	"errors"
	"fmt"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"net/http"
	"strings"
	"sync"
)

// ServerFactory is a fake server for instances of the armdomainservices.ClientFactory type.
type ServerFactory struct {
	Server                        Server
	DomainServiceOperationsServer DomainServiceOperationsServer
	OuContainerServer             OuContainerServer
	OuContainerOperationsServer   OuContainerOperationsServer
}

// NewServerFactoryTransport creates a new instance of ServerFactoryTransport with the provided implementation.
// The returned ServerFactoryTransport instance is connected to an instance of armdomainservices.ClientFactory via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewServerFactoryTransport(srv *ServerFactory) *ServerFactoryTransport {
	return &ServerFactoryTransport{
		srv: srv,
	}
}

// ServerFactoryTransport connects instances of armdomainservices.ClientFactory to instances of ServerFactory.
// Don't use this type directly, use NewServerFactoryTransport instead.
type ServerFactoryTransport struct {
	srv                             *ServerFactory
	trMu                            sync.Mutex
	trServer                        *ServerTransport
	trDomainServiceOperationsServer *DomainServiceOperationsServerTransport
	trOuContainerServer             *OuContainerServerTransport
	trOuContainerOperationsServer   *OuContainerOperationsServerTransport
}

// Do implements the policy.Transporter interface for ServerFactoryTransport.
func (s *ServerFactoryTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	client := method[:strings.Index(method, ".")]
	var resp *http.Response
	var err error

	switch client {
	case "Client":
		initServer(s, &s.trServer, func() *ServerTransport { return NewServerTransport(&s.srv.Server) })
		resp, err = s.trServer.Do(req)
	case "DomainServiceOperationsClient":
		initServer(s, &s.trDomainServiceOperationsServer, func() *DomainServiceOperationsServerTransport {
			return NewDomainServiceOperationsServerTransport(&s.srv.DomainServiceOperationsServer)
		})
		resp, err = s.trDomainServiceOperationsServer.Do(req)
	case "OuContainerClient":
		initServer(s, &s.trOuContainerServer, func() *OuContainerServerTransport { return NewOuContainerServerTransport(&s.srv.OuContainerServer) })
		resp, err = s.trOuContainerServer.Do(req)
	case "OuContainerOperationsClient":
		initServer(s, &s.trOuContainerOperationsServer, func() *OuContainerOperationsServerTransport {
			return NewOuContainerOperationsServerTransport(&s.srv.OuContainerOperationsServer)
		})
		resp, err = s.trOuContainerOperationsServer.Do(req)
	default:
		err = fmt.Errorf("unhandled client %s", client)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func initServer[T any](s *ServerFactoryTransport, dst **T, src func() *T) {
	s.trMu.Lock()
	if *dst == nil {
		*dst = src()
	}
	s.trMu.Unlock()
}
