// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) Go Code Generator. DO NOT EDIT.

package armedgezones

import "time"

// ExtendedZone - Resource that represents an Azure Extended Zone available to a subscription for registering and unregistering.
type ExtendedZone struct {
	// The resource-specific properties for this resource.
	Properties *ExtendedZoneProperties

	// READ-ONLY; The name of the ExtendedZone
	Name *string

	// READ-ONLY; Fully qualified resource ID for the resource. Ex - /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}
	ID *string

	// READ-ONLY; Azure Resource Manager metadata containing createdBy and modifiedBy information.
	SystemData *SystemData

	// READ-ONLY; The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or "Microsoft.Storage/storageAccounts"
	Type *string
}

// ExtendedZoneListResult - The response of a ExtendedZone list operation.
type ExtendedZoneListResult struct {
	// REQUIRED; The ExtendedZone items on this page
	Value []*ExtendedZone

	// The link to the next page of items
	NextLink *string
}

// ExtendedZoneProperties - The properties of an Extended Zone resource.
type ExtendedZoneProperties struct {
	// READ-ONLY; Display name of the Azure Extended Zone.
	DisplayName *string

	// READ-ONLY; Geography of the Azure Extended Zone.
	Geography *string

	// READ-ONLY; The Geography Group of the Azure Extended Zone.
	GeographyGroup *string

	// READ-ONLY; The Home Location of the Azure Extended Zone.
	HomeLocation *string

	// READ-ONLY; The Latitude of the Azure Extended Zone.
	Latitude *string

	// READ-ONLY; The Longitude of the Azure Extended Zone.
	Longitude *string

	// READ-ONLY; Category of region for the Azure Extended Zone.
	RegionCategory *string

	// READ-ONLY; Type of region for the Azure Extended Zone.
	RegionType *string

	// READ-ONLY; Regional display name of the Azure Extended Zone.
	RegionalDisplayName *string

	// READ-ONLY; Status of the last operation performed by the subscription on the Edge Zone resource
	ProvisioningState *ProvisioningState

	// READ-ONLY; Indicates the Azure Extended Zone registration’s approval status.
	RegistrationState *RegistrationState
}

// Operation - Details of a REST API operation, returned from the Resource Provider Operations API
type Operation struct {
	// Extensible enum. Indicates the action type. "Internal" refers to actions that are for internal only APIs.
	ActionType *ActionType

	// Localized display information for this particular operation.
	Display *OperationDisplay

	// READ-ONLY; Whether the operation applies to data-plane. This is "true" for data-plane operations and "false" for Azure
	// Resource Manager/control-plane operations.
	IsDataAction *bool

	// READ-ONLY; The name of the operation, as per Resource-Based Access Control (RBAC). Examples: "Microsoft.Compute/virtualMachines/write",
	// "Microsoft.Compute/virtualMachines/capture/action"
	Name *string

	// READ-ONLY; The intended executor of the operation; as in Resource Based Access Control (RBAC) and audit logs UX. Default
	// value is "user,system"
	Origin *Origin
}

// OperationDisplay - Localized display information for and operation.
type OperationDisplay struct {
	// The short, localized friendly description of the operation; suitable for tool tips and detailed views.
	Description *string

	// The concise, localized friendly name for the operation; suitable for dropdowns. E.g. "Create or Update Virtual Machine",
	// "Restart Virtual Machine".
	Operation *string

	// The localized friendly form of the resource provider name, e.g. "Microsoft Monitoring Insights" or "Microsoft Compute".
	Provider *string

	// The localized friendly name of the resource type related to this operation. E.g. "Virtual Machines" or "Job Schedule Collections".
	Resource *string
}

// OperationListResult - A list of REST API operations supported by an Azure Resource Provider. It contains an URL link to
// get the next set of results.
type OperationListResult struct {
	// REQUIRED; The Operation items on this page
	Value []*Operation

	// The link to the next page of items
	NextLink *string
}

// SystemData - Metadata pertaining to creation and last modification of the resource.
type SystemData struct {
	// The timestamp of resource creation (UTC).
	CreatedAt *time.Time

	// The identity that created the resource.
	CreatedBy *string

	// The type of identity that created the resource.
	CreatedByType *CreatedByType

	// The timestamp of resource last modification (UTC)
	LastModifiedAt *time.Time

	// The identity that last modified the resource.
	LastModifiedBy *string

	// The type of identity that last modified the resource.
	LastModifiedByType *CreatedByType
}
