// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) Go Code Generator. DO NOT EDIT.

package armfabric_test

import (
	"context"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/to"
	"github.com/Azure/azure-sdk-for-go/sdk/azidentity"
	"github.com/Azure/azure-sdk-for-go/sdk/resourcemanager/fabric/armfabric"
	"log"
)

// Generated from example definition: 2023-11-01/FabricCapacities_CheckNameAvailability.json
func ExampleCapacitiesClient_CheckNameAvailability() {
	cred, err := azidentity.NewDefaultAzureCredential(nil)
	if err != nil {
		log.Fatalf("failed to obtain a credential: %v", err)
	}
	ctx := context.Background()
	clientFactory, err := armfabric.NewClientFactory("548B7FB7-3B2A-4F46-BB02-66473F1FC22C", cred, nil)
	if err != nil {
		log.Fatalf("failed to create client: %v", err)
	}
	res, err := clientFactory.NewCapacitiesClient().CheckNameAvailability(ctx, "westcentralus", armfabric.CheckNameAvailabilityRequest{
		Name: to.Ptr("azsdktest"),
		Type: to.Ptr("Microsoft.Fabric/capacities"),
	}, nil)
	if err != nil {
		log.Fatalf("failed to finish the request: %v", err)
	}
	// You could use response here. We use blank identifier for just demo purposes.
	_ = res
	// If the HTTP response code is 200 as defined in example definition, your response structure would look as follows. Please pay attention that all the values in the output are fake values for just demo purposes.
	// res = armfabric.CapacitiesClientCheckNameAvailabilityResponse{
	// 	CheckNameAvailabilityResponse: &armfabric.CheckNameAvailabilityResponse{
	// 		NameAvailable: to.Ptr(true),
	// 	},
	// }
}

// Generated from example definition: 2023-11-01/FabricCapacities_CreateOrUpdate.json
func ExampleCapacitiesClient_BeginCreateOrUpdate() {
	cred, err := azidentity.NewDefaultAzureCredential(nil)
	if err != nil {
		log.Fatalf("failed to obtain a credential: %v", err)
	}
	ctx := context.Background()
	clientFactory, err := armfabric.NewClientFactory("548B7FB7-3B2A-4F46-BB02-66473F1FC22C", cred, nil)
	if err != nil {
		log.Fatalf("failed to create client: %v", err)
	}
	poller, err := clientFactory.NewCapacitiesClient().BeginCreateOrUpdate(ctx, "TestRG", "azsdktest", armfabric.Capacity{
		Properties: &armfabric.CapacityProperties{
			Administration: &armfabric.CapacityAdministration{
				Members: []*string{
					to.Ptr("azsdktest@microsoft.com"),
					to.Ptr("azsdktest2@microsoft.com"),
				},
			},
		},
		SKU: &armfabric.RpSKU{
			Name: to.Ptr("F2"),
			Tier: to.Ptr(armfabric.RpSKUTierFabric),
		},
		Location: to.Ptr("westcentralus"),
	}, nil)
	if err != nil {
		log.Fatalf("failed to finish the request: %v", err)
	}
	res, err := poller.PollUntilDone(ctx, nil)
	if err != nil {
		log.Fatalf("failed to pull the result: %v", err)
	}
	// You could use response here. We use blank identifier for just demo purposes.
	_ = res
	// If the HTTP response code is 200 as defined in example definition, your response structure would look as follows. Please pay attention that all the values in the output are fake values for just demo purposes.
	// res = armfabric.CapacitiesClientCreateOrUpdateResponse{
	// 	Capacity: &armfabric.Capacity{
	// 		Properties: &armfabric.CapacityProperties{
	// 			ProvisioningState: to.Ptr(armfabric.ProvisioningStateUpdating),
	// 			State: to.Ptr(armfabric.ResourceStatePreparing),
	// 			Administration: &armfabric.CapacityAdministration{
	// 				Members: []*string{
	// 					to.Ptr("azsdktest@microsoft.com"),
	// 					to.Ptr("azsdktest2@microsoft.com"),
	// 				},
	// 			},
	// 		},
	// 		SKU: &armfabric.RpSKU{
	// 			Name: to.Ptr("F2"),
	// 			Tier: to.Ptr(armfabric.RpSKUTierFabric),
	// 		},
	// 		Tags: map[string]*string{
	// 			"testKey": to.Ptr("testValue"),
	// 		},
	// 		Location: to.Ptr("westcentralus"),
	// 		ID: to.Ptr("/subscriptions/548B7FB7-3B2A-4F46-BB02-66473F1FC22C/resourceGroups/TestRG/providers/Microsoft.Fabric/capacities/azsdktest"),
	// 		Name: to.Ptr("azsdktest"),
	// 		Type: to.Ptr("Microsoft.Fabric/capacities"),
	// 	},
	// }
}

// Generated from example definition: 2023-11-01/FabricCapacities_Delete.json
func ExampleCapacitiesClient_BeginDelete() {
	cred, err := azidentity.NewDefaultAzureCredential(nil)
	if err != nil {
		log.Fatalf("failed to obtain a credential: %v", err)
	}
	ctx := context.Background()
	clientFactory, err := armfabric.NewClientFactory("548B7FB7-3B2A-4F46-BB02-66473F1FC22C", cred, nil)
	if err != nil {
		log.Fatalf("failed to create client: %v", err)
	}
	poller, err := clientFactory.NewCapacitiesClient().BeginDelete(ctx, "TestRG", "azsdktest", nil)
	if err != nil {
		log.Fatalf("failed to finish the request: %v", err)
	}
	_, err = poller.PollUntilDone(ctx, nil)
	if err != nil {
		log.Fatalf("failed to pull the result: %v", err)
	}
}

// Generated from example definition: 2023-11-01/FabricCapacities_Get.json
func ExampleCapacitiesClient_Get() {
	cred, err := azidentity.NewDefaultAzureCredential(nil)
	if err != nil {
		log.Fatalf("failed to obtain a credential: %v", err)
	}
	ctx := context.Background()
	clientFactory, err := armfabric.NewClientFactory("548B7FB7-3B2A-4F46-BB02-66473F1FC22C", cred, nil)
	if err != nil {
		log.Fatalf("failed to create client: %v", err)
	}
	res, err := clientFactory.NewCapacitiesClient().Get(ctx, "TestRG", "azsdktest", nil)
	if err != nil {
		log.Fatalf("failed to finish the request: %v", err)
	}
	// You could use response here. We use blank identifier for just demo purposes.
	_ = res
	// If the HTTP response code is 200 as defined in example definition, your response structure would look as follows. Please pay attention that all the values in the output are fake values for just demo purposes.
	// res = armfabric.CapacitiesClientGetResponse{
	// 	Capacity: &armfabric.Capacity{
	// 		Properties: &armfabric.CapacityProperties{
	// 			ProvisioningState: to.Ptr(armfabric.ProvisioningStateSucceeded),
	// 			State: to.Ptr(armfabric.ResourceStateActive),
	// 			Administration: &armfabric.CapacityAdministration{
	// 				Members: []*string{
	// 					to.Ptr("azsdktest@microsoft.com"),
	// 					to.Ptr("azsdktest2@microsoft.com"),
	// 				},
	// 			},
	// 		},
	// 		SKU: &armfabric.RpSKU{
	// 			Name: to.Ptr("F2"),
	// 			Tier: to.Ptr(armfabric.RpSKUTierFabric),
	// 		},
	// 		Tags: map[string]*string{
	// 		},
	// 		Location: to.Ptr("West Central US"),
	// 		ID: to.Ptr("/subscriptions/548B7FB7-3B2A-4F46-BB02-66473F1FC22C/resourceGroups/TestRG/providers/Microsoft.Fabric/capacities/azsdktest"),
	// 		Name: to.Ptr("azsdktest"),
	// 		Type: to.Ptr("Microsoft.Fabric/capacities"),
	// 	},
	// }
}

// Generated from example definition: 2023-11-01/FabricCapacities_ListByResourceGroup.json
func ExampleCapacitiesClient_NewListByResourceGroupPager() {
	cred, err := azidentity.NewDefaultAzureCredential(nil)
	if err != nil {
		log.Fatalf("failed to obtain a credential: %v", err)
	}
	ctx := context.Background()
	clientFactory, err := armfabric.NewClientFactory("548B7FB7-3B2A-4F46-BB02-66473F1FC22C", cred, nil)
	if err != nil {
		log.Fatalf("failed to create client: %v", err)
	}
	pager := clientFactory.NewCapacitiesClient().NewListByResourceGroupPager("TestRG", nil)
	for pager.More() {
		page, err := pager.NextPage(ctx)
		if err != nil {
			log.Fatalf("failed to advance page: %v", err)
		}
		for _, v := range page.Value {
			// You could use page here. We use blank identifier for just demo purposes.
			_ = v
		}
		// If the HTTP response code is 200 as defined in example definition, your page structure would look as follows. Please pay attention that all the values in the output are fake values for just demo purposes.
		// page = armfabric.CapacitiesClientListByResourceGroupResponse{
		// 	CapacityListResult: armfabric.CapacityListResult{
		// 		Value: []*armfabric.Capacity{
		// 			{
		// 				Properties: &armfabric.CapacityProperties{
		// 					ProvisioningState: to.Ptr(armfabric.ProvisioningStateSucceeded),
		// 					State: to.Ptr(armfabric.ResourceStateActive),
		// 					Administration: &armfabric.CapacityAdministration{
		// 						Members: []*string{
		// 							to.Ptr("azsdktest@microsoft.com"),
		// 						},
		// 					},
		// 				},
		// 				SKU: &armfabric.RpSKU{
		// 					Name: to.Ptr("F2"),
		// 					Tier: to.Ptr(armfabric.RpSKUTierFabric),
		// 				},
		// 				Tags: map[string]*string{
		// 				},
		// 				Location: to.Ptr("West Central US"),
		// 				ID: to.Ptr("/subscriptions/548B7FB7-3B2A-4F46-BB02-66473F1FC22C/resourceGroups/TestRG/providers/Microsoft.Fabric/capacities/azsdktest"),
		// 				Name: to.Ptr("azsdktest"),
		// 				Type: to.Ptr("Microsoft.Fabric/capacities"),
		// 			},
		// 			{
		// 				Properties: &armfabric.CapacityProperties{
		// 					ProvisioningState: to.Ptr(armfabric.ProvisioningStateProvisioning),
		// 					State: to.Ptr(armfabric.ResourceStateProvisioning),
		// 					Administration: &armfabric.CapacityAdministration{
		// 						Members: []*string{
		// 							to.Ptr("azsdktest@microsoft.com"),
		// 						},
		// 					},
		// 				},
		// 				SKU: &armfabric.RpSKU{
		// 					Name: to.Ptr("F4"),
		// 					Tier: to.Ptr(armfabric.RpSKUTierFabric),
		// 				},
		// 				Tags: map[string]*string{
		// 					"testKey": to.Ptr("testValue"),
		// 				},
		// 				Location: to.Ptr("West Central US"),
		// 				ID: to.Ptr("/subscriptions/548B7FB7-3B2A-4F46-BB02-66473F1FC22C/resourceGroups/TestRG/providers/Microsoft.Fabric/capacities/azsdktest2"),
		// 				Name: to.Ptr("azsdktest2"),
		// 				Type: to.Ptr("Microsoft.Fabric/capacities"),
		// 			},
		// 		},
		// 	},
		// }
	}
}

// Generated from example definition: 2023-11-01/FabricCapacities_ListBySubscription.json
func ExampleCapacitiesClient_NewListBySubscriptionPager() {
	cred, err := azidentity.NewDefaultAzureCredential(nil)
	if err != nil {
		log.Fatalf("failed to obtain a credential: %v", err)
	}
	ctx := context.Background()
	clientFactory, err := armfabric.NewClientFactory("548B7FB7-3B2A-4F46-BB02-66473F1FC22C", cred, nil)
	if err != nil {
		log.Fatalf("failed to create client: %v", err)
	}
	pager := clientFactory.NewCapacitiesClient().NewListBySubscriptionPager(nil)
	for pager.More() {
		page, err := pager.NextPage(ctx)
		if err != nil {
			log.Fatalf("failed to advance page: %v", err)
		}
		for _, v := range page.Value {
			// You could use page here. We use blank identifier for just demo purposes.
			_ = v
		}
		// If the HTTP response code is 200 as defined in example definition, your page structure would look as follows. Please pay attention that all the values in the output are fake values for just demo purposes.
		// page = armfabric.CapacitiesClientListBySubscriptionResponse{
		// 	CapacityListResult: armfabric.CapacityListResult{
		// 		Value: []*armfabric.Capacity{
		// 			{
		// 				Properties: &armfabric.CapacityProperties{
		// 					ProvisioningState: to.Ptr(armfabric.ProvisioningStateSucceeded),
		// 					State: to.Ptr(armfabric.ResourceStateActive),
		// 					Administration: &armfabric.CapacityAdministration{
		// 						Members: []*string{
		// 							to.Ptr("azsdktest@microsoft.com"),
		// 						},
		// 					},
		// 				},
		// 				SKU: &armfabric.RpSKU{
		// 					Name: to.Ptr("F2"),
		// 					Tier: to.Ptr(armfabric.RpSKUTierFabric),
		// 				},
		// 				Tags: map[string]*string{
		// 				},
		// 				Location: to.Ptr("West Central US"),
		// 				ID: to.Ptr("/subscriptions/548B7FB7-3B2A-4F46-BB02-66473F1FC22C/resourceGroups/TestRG/providers/Microsoft.Fabric/capacities/azsdktest"),
		// 				Name: to.Ptr("azsdktest"),
		// 				Type: to.Ptr("Microsoft.Fabric/capacities"),
		// 			},
		// 			{
		// 				Properties: &armfabric.CapacityProperties{
		// 					ProvisioningState: to.Ptr(armfabric.ProvisioningStateProvisioning),
		// 					State: to.Ptr(armfabric.ResourceStateProvisioning),
		// 					Administration: &armfabric.CapacityAdministration{
		// 						Members: []*string{
		// 							to.Ptr("azsdktest@microsoft.com"),
		// 						},
		// 					},
		// 				},
		// 				SKU: &armfabric.RpSKU{
		// 					Name: to.Ptr("F4"),
		// 					Tier: to.Ptr(armfabric.RpSKUTierFabric),
		// 				},
		// 				Tags: map[string]*string{
		// 					"testKey": to.Ptr("testValue"),
		// 				},
		// 				Location: to.Ptr("West Central US"),
		// 				ID: to.Ptr("/subscriptions/548B7FB7-3B2A-4F46-BB02-66473F1FC22C/resourceGroups/TestRG/providers/Microsoft.Fabric/capacities/azsdktest2"),
		// 				Name: to.Ptr("azsdktest2"),
		// 				Type: to.Ptr("Microsoft.Fabric/capacities"),
		// 			},
		// 		},
		// 	},
		// }
	}
}

// Generated from example definition: 2023-11-01/FabricCapacities_ListSkus.json
func ExampleCapacitiesClient_NewListSKUsPager() {
	cred, err := azidentity.NewDefaultAzureCredential(nil)
	if err != nil {
		log.Fatalf("failed to obtain a credential: %v", err)
	}
	ctx := context.Background()
	clientFactory, err := armfabric.NewClientFactory("548B7FB7-3B2A-4F46-BB02-66473F1FC22C", cred, nil)
	if err != nil {
		log.Fatalf("failed to create client: %v", err)
	}
	pager := clientFactory.NewCapacitiesClient().NewListSKUsPager(nil)
	for pager.More() {
		page, err := pager.NextPage(ctx)
		if err != nil {
			log.Fatalf("failed to advance page: %v", err)
		}
		for _, v := range page.Value {
			// You could use page here. We use blank identifier for just demo purposes.
			_ = v
		}
		// If the HTTP response code is 200 as defined in example definition, your page structure would look as follows. Please pay attention that all the values in the output are fake values for just demo purposes.
		// page = armfabric.CapacitiesClientListSKUsResponse{
		// 	RpSKUEnumerationForNewResourceResult: armfabric.RpSKUEnumerationForNewResourceResult{
		// 		Value: []*armfabric.RpSKUDetailsForNewResource{
		// 			{
		// 				Name: to.Ptr("F8"),
		// 				Locations: []*string{
		// 					to.Ptr("West Europe"),
		// 				},
		// 				ResourceType: to.Ptr("Capacities"),
		// 			},
		// 			{
		// 				Name: to.Ptr("F64"),
		// 				Locations: []*string{
		// 					to.Ptr("West Europe"),
		// 				},
		// 				ResourceType: to.Ptr("Capacities"),
		// 			},
		// 			{
		// 				Name: to.Ptr("F128"),
		// 				Locations: []*string{
		// 					to.Ptr("West Europe"),
		// 				},
		// 				ResourceType: to.Ptr("Capacities"),
		// 			},
		// 			{
		// 				Name: to.Ptr("F512"),
		// 				Locations: []*string{
		// 					to.Ptr("West Europe"),
		// 				},
		// 				ResourceType: to.Ptr("Capacities"),
		// 			},
		// 		},
		// 	},
		// }
	}
}

// Generated from example definition: 2023-11-01/FabricCapacities_ListSkusForCapacity.json
func ExampleCapacitiesClient_NewListSKUsForCapacityPager() {
	cred, err := azidentity.NewDefaultAzureCredential(nil)
	if err != nil {
		log.Fatalf("failed to obtain a credential: %v", err)
	}
	ctx := context.Background()
	clientFactory, err := armfabric.NewClientFactory("548B7FB7-3B2A-4F46-BB02-66473F1FC22C", cred, nil)
	if err != nil {
		log.Fatalf("failed to create client: %v", err)
	}
	pager := clientFactory.NewCapacitiesClient().NewListSKUsForCapacityPager("TestRG", "azsdktest", nil)
	for pager.More() {
		page, err := pager.NextPage(ctx)
		if err != nil {
			log.Fatalf("failed to advance page: %v", err)
		}
		for _, v := range page.Value {
			// You could use page here. We use blank identifier for just demo purposes.
			_ = v
		}
		// If the HTTP response code is 200 as defined in example definition, your page structure would look as follows. Please pay attention that all the values in the output are fake values for just demo purposes.
		// page = armfabric.CapacitiesClientListSKUsForCapacityResponse{
		// 	RpSKUEnumerationForExistingResourceResult: armfabric.RpSKUEnumerationForExistingResourceResult{
		// 		Value: []*armfabric.RpSKUDetailsForExistingResource{
		// 			{
		// 				ResourceType: to.Ptr("Microsoft.Fabric/capacities"),
		// 				SKU: &armfabric.RpSKU{
		// 					Name: to.Ptr("F16"),
		// 					Tier: to.Ptr(armfabric.RpSKUTierFabric),
		// 				},
		// 			},
		// 			{
		// 				ResourceType: to.Ptr("Microsoft.Fabric/capacities"),
		// 				SKU: &armfabric.RpSKU{
		// 					Name: to.Ptr("F8"),
		// 					Tier: to.Ptr(armfabric.RpSKUTierFabric),
		// 				},
		// 			},
		// 			{
		// 				ResourceType: to.Ptr("Microsoft.Fabric/capacities"),
		// 				SKU: &armfabric.RpSKU{
		// 					Name: to.Ptr("F64"),
		// 					Tier: to.Ptr(armfabric.RpSKUTierFabric),
		// 				},
		// 			},
		// 			{
		// 				ResourceType: to.Ptr("Microsoft.Fabric/capacities"),
		// 				SKU: &armfabric.RpSKU{
		// 					Name: to.Ptr("F1024"),
		// 					Tier: to.Ptr(armfabric.RpSKUTierFabric),
		// 				},
		// 			},
		// 			{
		// 				ResourceType: to.Ptr("Microsoft.Fabric/capacities"),
		// 				SKU: &armfabric.RpSKU{
		// 					Name: to.Ptr("F128"),
		// 					Tier: to.Ptr(armfabric.RpSKUTierFabric),
		// 				},
		// 			},
		// 			{
		// 				ResourceType: to.Ptr("Microsoft.Fabric/capacities"),
		// 				SKU: &armfabric.RpSKU{
		// 					Name: to.Ptr("F2"),
		// 					Tier: to.Ptr(armfabric.RpSKUTierFabric),
		// 				},
		// 			},
		// 			{
		// 				ResourceType: to.Ptr("Microsoft.Fabric/capacities"),
		// 				SKU: &armfabric.RpSKU{
		// 					Name: to.Ptr("F256"),
		// 					Tier: to.Ptr(armfabric.RpSKUTierFabric),
		// 				},
		// 			},
		// 			{
		// 				ResourceType: to.Ptr("Microsoft.Fabric/capacities"),
		// 				SKU: &armfabric.RpSKU{
		// 					Name: to.Ptr("F32"),
		// 					Tier: to.Ptr(armfabric.RpSKUTierFabric),
		// 				},
		// 			},
		// 			{
		// 				ResourceType: to.Ptr("Microsoft.Fabric/capacities"),
		// 				SKU: &armfabric.RpSKU{
		// 					Name: to.Ptr("F4"),
		// 					Tier: to.Ptr(armfabric.RpSKUTierFabric),
		// 				},
		// 			},
		// 			{
		// 				ResourceType: to.Ptr("Microsoft.Fabric/capacities"),
		// 				SKU: &armfabric.RpSKU{
		// 					Name: to.Ptr("F512"),
		// 					Tier: to.Ptr(armfabric.RpSKUTierFabric),
		// 				},
		// 			},
		// 			{
		// 				ResourceType: to.Ptr("Microsoft.Fabric/capacities"),
		// 				SKU: &armfabric.RpSKU{
		// 					Name: to.Ptr("F2048"),
		// 					Tier: to.Ptr(armfabric.RpSKUTierFabric),
		// 				},
		// 			},
		// 		},
		// 	},
		// }
	}
}

// Generated from example definition: 2023-11-01/FabricCapacities_Resume.json
func ExampleCapacitiesClient_BeginResume() {
	cred, err := azidentity.NewDefaultAzureCredential(nil)
	if err != nil {
		log.Fatalf("failed to obtain a credential: %v", err)
	}
	ctx := context.Background()
	clientFactory, err := armfabric.NewClientFactory("548B7FB7-3B2A-4F46-BB02-66473F1FC22C", cred, nil)
	if err != nil {
		log.Fatalf("failed to create client: %v", err)
	}
	poller, err := clientFactory.NewCapacitiesClient().BeginResume(ctx, "TestRG", "azsdktest", nil)
	if err != nil {
		log.Fatalf("failed to finish the request: %v", err)
	}
	res, err := poller.PollUntilDone(ctx, nil)
	if err != nil {
		log.Fatalf("failed to pull the result: %v", err)
	}
	// You could use response here. We use blank identifier for just demo purposes.
	_ = res
	// If the HTTP response code is 200 as defined in example definition, your response structure would look as follows. Please pay attention that all the values in the output are fake values for just demo purposes.
	// res = armfabric.CapacitiesClientResumeResponse{
	// }
}

// Generated from example definition: 2023-11-01/FabricCapacities_Suspend.json
func ExampleCapacitiesClient_BeginSuspend() {
	cred, err := azidentity.NewDefaultAzureCredential(nil)
	if err != nil {
		log.Fatalf("failed to obtain a credential: %v", err)
	}
	ctx := context.Background()
	clientFactory, err := armfabric.NewClientFactory("548B7FB7-3B2A-4F46-BB02-66473F1FC22C", cred, nil)
	if err != nil {
		log.Fatalf("failed to create client: %v", err)
	}
	poller, err := clientFactory.NewCapacitiesClient().BeginSuspend(ctx, "TestRG", "azsdktest", nil)
	if err != nil {
		log.Fatalf("failed to finish the request: %v", err)
	}
	res, err := poller.PollUntilDone(ctx, nil)
	if err != nil {
		log.Fatalf("failed to pull the result: %v", err)
	}
	// You could use response here. We use blank identifier for just demo purposes.
	_ = res
	// If the HTTP response code is 200 as defined in example definition, your response structure would look as follows. Please pay attention that all the values in the output are fake values for just demo purposes.
	// res = armfabric.CapacitiesClientSuspendResponse{
	// }
}

// Generated from example definition: 2023-11-01/FabricCapacities_Update.json
func ExampleCapacitiesClient_BeginUpdate() {
	cred, err := azidentity.NewDefaultAzureCredential(nil)
	if err != nil {
		log.Fatalf("failed to obtain a credential: %v", err)
	}
	ctx := context.Background()
	clientFactory, err := armfabric.NewClientFactory("548B7FB7-3B2A-4F46-BB02-66473F1FC22C", cred, nil)
	if err != nil {
		log.Fatalf("failed to create client: %v", err)
	}
	poller, err := clientFactory.NewCapacitiesClient().BeginUpdate(ctx, "TestRG", "azsdktest", armfabric.CapacityUpdate{
		SKU: &armfabric.RpSKU{
			Name: to.Ptr("F8"),
			Tier: to.Ptr(armfabric.RpSKUTierFabric),
		},
		Tags: map[string]*string{
			"testKey": to.Ptr("testValue"),
		},
		Properties: &armfabric.CapacityUpdateProperties{
			Administration: &armfabric.CapacityAdministration{
				Members: []*string{
					to.Ptr("azsdktest2@microsoft.com"),
				},
			},
		},
	}, nil)
	if err != nil {
		log.Fatalf("failed to finish the request: %v", err)
	}
	res, err := poller.PollUntilDone(ctx, nil)
	if err != nil {
		log.Fatalf("failed to pull the result: %v", err)
	}
	// You could use response here. We use blank identifier for just demo purposes.
	_ = res
	// If the HTTP response code is 200 as defined in example definition, your response structure would look as follows. Please pay attention that all the values in the output are fake values for just demo purposes.
	// res = armfabric.CapacitiesClientUpdateResponse{
	// 	Capacity: &armfabric.Capacity{
	// 		Properties: &armfabric.CapacityProperties{
	// 			ProvisioningState: to.Ptr(armfabric.ProvisioningStateUpdating),
	// 			State: to.Ptr(armfabric.ResourceStatePreparing),
	// 			Administration: &armfabric.CapacityAdministration{
	// 				Members: []*string{
	// 					to.Ptr("azsdktest2@microsoft.com"),
	// 				},
	// 			},
	// 		},
	// 		SKU: &armfabric.RpSKU{
	// 			Name: to.Ptr("F8"),
	// 			Tier: to.Ptr(armfabric.RpSKUTierFabric),
	// 		},
	// 		Tags: map[string]*string{
	// 			"testKey": to.Ptr("testValue"),
	// 		},
	// 		Location: to.Ptr("westcentralus"),
	// 		ID: to.Ptr("/subscriptions/548B7FB7-3B2A-4F46-BB02-66473F1FC22C/resourceGroups/TestRG/providers/Microsoft.Fabric/capacities/azsdktest"),
	// 		Name: to.Ptr("azsdktest"),
	// 		Type: to.Ptr("Microsoft.Fabric/capacities"),
	// 	},
	// }
}
