// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) Go Code Generator. DO NOT EDIT.

package armfabric

const (
	moduleName    = "github.com/Azure/azure-sdk-for-go/sdk/resourcemanager/fabric/armfabric"
	moduleVersion = "v1.0.0"
)

// ActionType - Extensible enum. Indicates the action type. "Internal" refers to actions that are for internal only APIs.
type ActionType string

const (
	// ActionTypeInternal - Actions are for internal-only APIs.
	ActionTypeInternal ActionType = "Internal"
)

// PossibleActionTypeValues returns the possible values for the ActionType const type.
func PossibleActionTypeValues() []ActionType {
	return []ActionType{
		ActionTypeInternal,
	}
}

// CheckNameAvailabilityReason - Possible reasons for a name not being available.
type CheckNameAvailabilityReason string

const (
	// CheckNameAvailabilityReasonAlreadyExists - Name already exists.
	CheckNameAvailabilityReasonAlreadyExists CheckNameAvailabilityReason = "AlreadyExists"
	// CheckNameAvailabilityReasonInvalid - Name is invalid.
	CheckNameAvailabilityReasonInvalid CheckNameAvailabilityReason = "Invalid"
)

// PossibleCheckNameAvailabilityReasonValues returns the possible values for the CheckNameAvailabilityReason const type.
func PossibleCheckNameAvailabilityReasonValues() []CheckNameAvailabilityReason {
	return []CheckNameAvailabilityReason{
		CheckNameAvailabilityReasonAlreadyExists,
		CheckNameAvailabilityReasonInvalid,
	}
}

// CreatedByType - The kind of entity that created the resource.
type CreatedByType string

const (
	// CreatedByTypeApplication - The entity was created by an application.
	CreatedByTypeApplication CreatedByType = "Application"
	// CreatedByTypeKey - The entity was created by a key.
	CreatedByTypeKey CreatedByType = "Key"
	// CreatedByTypeManagedIdentity - The entity was created by a managed identity.
	CreatedByTypeManagedIdentity CreatedByType = "ManagedIdentity"
	// CreatedByTypeUser - The entity was created by a user.
	CreatedByTypeUser CreatedByType = "User"
)

// PossibleCreatedByTypeValues returns the possible values for the CreatedByType const type.
func PossibleCreatedByTypeValues() []CreatedByType {
	return []CreatedByType{
		CreatedByTypeApplication,
		CreatedByTypeKey,
		CreatedByTypeManagedIdentity,
		CreatedByTypeUser,
	}
}

// Origin - The intended executor of the operation; as in Resource Based Access Control (RBAC) and audit logs UX. Default
// value is "user,system"
type Origin string

const (
	// OriginSystem - Indicates the operation is initiated by a system.
	OriginSystem Origin = "system"
	// OriginUser - Indicates the operation is initiated by a user.
	OriginUser Origin = "user"
	// OriginUserSystem - Indicates the operation is initiated by a user or system.
	OriginUserSystem Origin = "user,system"
)

// PossibleOriginValues returns the possible values for the Origin const type.
func PossibleOriginValues() []Origin {
	return []Origin{
		OriginSystem,
		OriginUser,
		OriginUserSystem,
	}
}

// ProvisioningState - The provisioning state of the Fabric capacity resource.
type ProvisioningState string

const (
	// ProvisioningStateCanceled - Resource creation was canceled.
	ProvisioningStateCanceled ProvisioningState = "Canceled"
	// ProvisioningStateDeleting - Resource is deleting
	ProvisioningStateDeleting ProvisioningState = "Deleting"
	// ProvisioningStateFailed - Resource creation failed.
	ProvisioningStateFailed ProvisioningState = "Failed"
	// ProvisioningStateProvisioning - Resource is provisioning
	ProvisioningStateProvisioning ProvisioningState = "Provisioning"
	// ProvisioningStateSucceeded - Resource has been created.
	ProvisioningStateSucceeded ProvisioningState = "Succeeded"
	// ProvisioningStateUpdating - Resource is updating
	ProvisioningStateUpdating ProvisioningState = "Updating"
)

// PossibleProvisioningStateValues returns the possible values for the ProvisioningState const type.
func PossibleProvisioningStateValues() []ProvisioningState {
	return []ProvisioningState{
		ProvisioningStateCanceled,
		ProvisioningStateDeleting,
		ProvisioningStateFailed,
		ProvisioningStateProvisioning,
		ProvisioningStateSucceeded,
		ProvisioningStateUpdating,
	}
}

// ResourceState - The state of the Fabric capacity resource.
type ResourceState string

const (
	// ResourceStateActive - Resource is active
	ResourceStateActive ResourceState = "Active"
	// ResourceStateDeleting - Resource is deleting
	ResourceStateDeleting ResourceState = "Deleting"
	// ResourceStateFailed - Resource is failed
	ResourceStateFailed ResourceState = "Failed"
	// ResourceStatePaused - Resource is paused
	ResourceStatePaused ResourceState = "Paused"
	// ResourceStatePausing - Resource is pausing
	ResourceStatePausing ResourceState = "Pausing"
	// ResourceStatePreparing - Resource is preparing
	ResourceStatePreparing ResourceState = "Preparing"
	// ResourceStateProvisioning - Resource is provisioning
	ResourceStateProvisioning ResourceState = "Provisioning"
	// ResourceStateResuming - Resource is resuming
	ResourceStateResuming ResourceState = "Resuming"
	// ResourceStateScaling - Resource is scaling
	ResourceStateScaling ResourceState = "Scaling"
	// ResourceStateSuspended - Resource is suspended
	ResourceStateSuspended ResourceState = "Suspended"
	// ResourceStateSuspending - Resource is suspending
	ResourceStateSuspending ResourceState = "Suspending"
	// ResourceStateUpdating - Resource is updating
	ResourceStateUpdating ResourceState = "Updating"
)

// PossibleResourceStateValues returns the possible values for the ResourceState const type.
func PossibleResourceStateValues() []ResourceState {
	return []ResourceState{
		ResourceStateActive,
		ResourceStateDeleting,
		ResourceStateFailed,
		ResourceStatePaused,
		ResourceStatePausing,
		ResourceStatePreparing,
		ResourceStateProvisioning,
		ResourceStateResuming,
		ResourceStateScaling,
		ResourceStateSuspended,
		ResourceStateSuspending,
		ResourceStateUpdating,
	}
}

// RpSKUTier - The name of the Azure pricing tier to which the SKU applies.
type RpSKUTier string

const (
	// RpSKUTierFabric - Fabric tier
	RpSKUTierFabric RpSKUTier = "Fabric"
)

// PossibleRpSKUTierValues returns the possible values for the RpSKUTier const type.
func PossibleRpSKUTierValues() []RpSKUTier {
	return []RpSKUTier{
		RpSKUTierFabric,
	}
}
