//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package armhardwaresecuritymodules

import "time"

// APIEntityReference - The API entity reference.
type APIEntityReference struct {
	// The Azure resource id in the form of /subscriptions/{SubscriptionId}/resourceGroups/{ResourceGroupName}/…
	ResourceID *string
}

// BackupRequestProperties - Backup properties
type BackupRequestProperties struct {
	// REQUIRED; The Azure blob storage container Uri which contains the backup
	AzureStorageBlobContainerURI *string

	// The SAS token pointing to an Azure blob storage container. This property is reserved for Azure Backup Service.
	Token *string
}

// BackupRestoreBaseResultProperties - Backup and Restore operation common properties
type BackupRestoreBaseResultProperties struct {
	// Error encountered, if any, during the backup/restore operation.
	Error *ErrorDetail

	// Identifier for the backup/restore operation.
	JobID *string

	// The status details of backup/restore operation
	StatusDetails *string

	// READ-ONLY; The end time of the backup/restore operation in UTC
	EndTime *time.Time

	// READ-ONLY; The start time of the backup/restore operation in UTC
	StartTime *time.Time

	// READ-ONLY; Status of the backup/restore operation
	Status *BackupRestoreOperationStatus
}

// BackupRestoreRequestBaseProperties - Backup and Restore Common properties
type BackupRestoreRequestBaseProperties struct {
	// REQUIRED; The Azure blob storage container Uri which contains the backup
	AzureStorageBlobContainerURI *string

	// The SAS token pointing to an Azure blob storage container. This property is reserved for Azure Backup Service.
	Token *string
}

// BackupResult - Backup operation Result
type BackupResult struct {
	// Properties of the Cloud HSM Cluster
	Properties *BackupResultProperties
}

// BackupResultProperties - Properties of the Cloud HSM Cluster
type BackupResultProperties struct {
	// The Azure blob storage container Uri which contains the backup
	AzureStorageBlobContainerURI *string

	// The ID of the backup.
	BackupID *string

	// Error encountered, if any, during the backup/restore operation.
	Error *ErrorDetail

	// Identifier for the backup/restore operation.
	JobID *string

	// The status details of backup/restore operation
	StatusDetails *string

	// READ-ONLY; The end time of the backup/restore operation in UTC
	EndTime *time.Time

	// READ-ONLY; The start time of the backup/restore operation in UTC
	StartTime *time.Time

	// READ-ONLY; Status of the backup/restore operation
	Status *BackupRestoreOperationStatus
}

// CloudHsmCluster - Resource information with extended details.
type CloudHsmCluster struct {
	// REQUIRED; The geo-location where the resource lives
	Location *string

	// Managed service identity (system assigned and/or user assigned identities)
	Identity *ManagedServiceIdentity

	// Properties of the Cloud HSM Cluster
	Properties *CloudHsmClusterProperties

	// SKU details
	SKU *CloudHsmClusterSKU

	// Resource tags.
	Tags map[string]*string

	// READ-ONLY; Fully qualified resource ID for the resource. E.g. "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}"
	ID *string

	// READ-ONLY; The name of the resource
	Name *string

	// READ-ONLY; Azure Resource Manager metadata containing createdBy and modifiedBy information.
	SystemData *SystemData

	// READ-ONLY; The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or "Microsoft.Storage/storageAccounts"
	Type *string
}

// CloudHsmClusterListResult - List of Cloud HSM Clusters
type CloudHsmClusterListResult struct {
	// The URL to get the next set of Cloud HSM Clusters.
	NextLink *string

	// The list of Cloud HSM Clusters.
	Value []*CloudHsmCluster
}

// CloudHsmClusterPatchParameters - Patchable properties of the Cloud HSM Cluster
type CloudHsmClusterPatchParameters struct {
	// Managed service identity (system assigned and/or user assigned identities)
	Identity *ManagedServiceIdentity

	// The Cloud HSM Cluster's tags
	Tags map[string]*string
}

// CloudHsmClusterProperties - Properties of a Cloud HSM Cluster.
type CloudHsmClusterProperties struct {
	// The Cloud HSM Cluster's auto-generated Domain Name Label Scope
	AutoGeneratedDomainNameLabelScope *AutoGeneratedDomainNameLabelScope

	// The Cloud HSM Cluster public network access
	PublicNetworkAccess *PublicNetworkAccess

	// READ-ONLY; State of security domain activation
	ActivationState *ActivationState

	// READ-ONLY; An array of Cloud HSM Cluster's HSMs
	Hsms []*CloudHsmProperties

	// READ-ONLY; List of private endpoint connection resources
	PrivateEndpointConnections []*PrivateEndpointConnection

	// READ-ONLY; The Cloud HSM Cluster's provisioningState
	ProvisioningState *ProvisioningState

	// READ-ONLY; Cloud HSM Cluster status message
	StatusMessage *string
}

// CloudHsmClusterResource - Cloud HSM Cluster Resource
type CloudHsmClusterResource struct {
	// REQUIRED; The geo-location where the resource lives
	Location *string

	// Managed service identity (system assigned and/or user assigned identities)
	Identity *ManagedServiceIdentity

	// SKU details
	SKU *CloudHsmClusterSKU

	// Resource tags.
	Tags map[string]*string

	// READ-ONLY; Fully qualified resource ID for the resource. E.g. "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}"
	ID *string

	// READ-ONLY; The name of the resource
	Name *string

	// READ-ONLY; Azure Resource Manager metadata containing createdBy and modifiedBy information.
	SystemData *SystemData

	// READ-ONLY; The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or "Microsoft.Storage/storageAccounts"
	Type *string
}

// CloudHsmClusterSKU - Cloud Hsm Cluster SKU information
type CloudHsmClusterSKU struct {
	// REQUIRED; Sku family of the Cloud HSM Cluster
	Family *CloudHsmClusterSKUFamily

	// REQUIRED; Sku name of the Cloud HSM Cluster
	Name *CloudHsmClusterSKUName

	// Sku capacity
	Capacity *int32
}

// CloudHsmProperties - The Cloud HSM Properties
type CloudHsmProperties struct {
	// FQDN of the Cloud HSM
	Fqdn *string

	// The Cloud HSM State. Values are: Deploying, ConfiguringSlb, Starting, Starting, Failed, Failed, Deleting, DeletingSlbEntry,
	// InitialProvisioning, Updating
	State *string

	// The Cloud HSM State message
	StateMessage *string
}

// DedicatedHsm - Resource information with extended details.
type DedicatedHsm struct {
	// REQUIRED; The geo-location where the resource lives
	Location *string

	// REQUIRED; Properties of the dedicated HSM
	Properties *DedicatedHsmProperties

	// REQUIRED; SKU details
	SKU *SKU

	// Resource tags.
	Tags map[string]*string

	// The Dedicated Hsm zones.
	Zones []*string

	// READ-ONLY; Fully qualified resource ID for the resource. E.g. "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}"
	ID *string

	// READ-ONLY; The name of the resource
	Name *string

	// READ-ONLY; Azure Resource Manager metadata containing createdBy and modifiedBy information.
	SystemData *SystemData

	// READ-ONLY; The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or "Microsoft.Storage/storageAccounts"
	Type *string
}

// DedicatedHsmError - The error exception.
type DedicatedHsmError struct {
	// READ-ONLY; The error detail of the operation if any.
	Error *Error
}

// DedicatedHsmListResult - List of dedicated HSMs
type DedicatedHsmListResult struct {
	// The URL to get the next set of dedicated hsms.
	NextLink *string

	// The list of dedicated HSMs.
	Value []*DedicatedHsm
}

// DedicatedHsmPatchParameters - Patchable properties of the dedicated HSM
type DedicatedHsmPatchParameters struct {
	// Resource tags
	Tags map[string]*string
}

// DedicatedHsmProperties - Properties of the dedicated hsm
type DedicatedHsmProperties struct {
	// Specifies the management network interfaces of the dedicated hsm.
	ManagementNetworkProfile *NetworkProfile

	// Specifies the network interfaces of the dedicated hsm.
	NetworkProfile *NetworkProfile

	// This field will be used when RP does not support Availability zones.
	StampID *string

	// READ-ONLY; Provisioning state.
	ProvisioningState *JSONWebKeyType

	// READ-ONLY; Resource Status Message.
	StatusMessage *string
}

// EndpointDependency - A domain name that dedicated hsm services are reaching at.
type EndpointDependency struct {
	// The domain name of the dependency.
	DomainName *string

	// The Ports and Protocols used when connecting to domainName.
	EndpointDetails []*EndpointDetail
}

// EndpointDetail - Connect information from the dedicated hsm service to a single endpoint.
type EndpointDetail struct {
	// Description of the detail
	Description *string

	// An IP Address that Domain Name currently resolves to.
	IPAddress *string

	// The port an endpoint is connected to.
	Port *int32

	// The protocol used for connection
	Protocol *string
}

// Error - The key vault server error.
type Error struct {
	// READ-ONLY; The error code.
	Code *string

	// READ-ONLY; Contains more specific error that narrows down the cause. May be null.
	InnerError *Error

	// READ-ONLY; The error message.
	Message *string
}

// ErrorAdditionalInfo - The resource management error additional info.
type ErrorAdditionalInfo struct {
	// READ-ONLY; The additional info.
	Info any

	// READ-ONLY; The additional info type.
	Type *string
}

// ErrorDetail - The error detail.
type ErrorDetail struct {
	// READ-ONLY; The error additional info.
	AdditionalInfo []*ErrorAdditionalInfo

	// READ-ONLY; The error code.
	Code *string

	// READ-ONLY; The error details.
	Details []*ErrorDetail

	// READ-ONLY; The error message.
	Message *string

	// READ-ONLY; The error target.
	Target *string
}

// ErrorResponse - Common error response for all Azure Resource Manager APIs to return error details for failed operations.
// (This also follows the OData error response format.).
type ErrorResponse struct {
	// The error object.
	Error *ErrorDetail
}

// ManagedServiceIdentity - Managed service identity (system assigned and/or user assigned identities)
type ManagedServiceIdentity struct {
	// REQUIRED; Type of managed service identity (where both SystemAssigned and UserAssigned types are allowed).
	Type *ManagedServiceIdentityType

	// The set of user assigned identities associated with the resource. The userAssignedIdentities dictionary keys will be ARM
	// resource ids in the form:
	// '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}.
	// The dictionary values can be empty objects ({}) in
	// requests.
	UserAssignedIdentities map[string]*UserAssignedIdentity

	// READ-ONLY; The service principal ID of the system assigned identity. This property will only be provided for a system assigned
	// identity.
	PrincipalID *string

	// READ-ONLY; The tenant ID of the system assigned identity. This property will only be provided for a system assigned identity.
	TenantID *string
}

// NetworkInterface - The network interface definition.
type NetworkInterface struct {
	// Private Ip address of the interface
	PrivateIPAddress *string

	// READ-ONLY; The Azure resource id in the form of /subscriptions/{SubscriptionId}/resourceGroups/{ResourceGroupName}/…
	ResourceID *string
}

// NetworkProfile - The network profile definition.
type NetworkProfile struct {
	// Specifies the list of resource Ids for the network interfaces associated with the dedicated HSM.
	NetworkInterfaces []*NetworkInterface

	// Specifies the identifier of the subnet.
	Subnet *APIEntityReference
}

// Operation - Details of a REST API operation, returned from the Resource Provider Operations API
type Operation struct {
	// Localized display information for this particular operation.
	Display *OperationDisplay

	// READ-ONLY; Enum. Indicates the action type. "Internal" refers to actions that are for internal only APIs.
	ActionType *ActionType

	// READ-ONLY; Whether the operation applies to data-plane. This is "true" for data-plane operations and "false" for ARM/control-plane
	// operations.
	IsDataAction *bool

	// READ-ONLY; The name of the operation, as per Resource-Based Access Control (RBAC). Examples: "Microsoft.Compute/virtualMachines/write",
	// "Microsoft.Compute/virtualMachines/capture/action"
	Name *string

	// READ-ONLY; The intended executor of the operation; as in Resource Based Access Control (RBAC) and audit logs UX. Default
	// value is "user,system"
	Origin *Origin
}

// OperationDisplay - Localized display information for this particular operation.
type OperationDisplay struct {
	// READ-ONLY; The short, localized friendly description of the operation; suitable for tool tips and detailed views.
	Description *string

	// READ-ONLY; The concise, localized friendly name for the operation; suitable for dropdowns. E.g. "Create or Update Virtual
	// Machine", "Restart Virtual Machine".
	Operation *string

	// READ-ONLY; The localized friendly form of the resource provider name, e.g. "Microsoft Monitoring Insights" or "Microsoft
	// Compute".
	Provider *string

	// READ-ONLY; The localized friendly name of the resource type related to this operation. E.g. "Virtual Machines" or "Job
	// Schedule Collections".
	Resource *string
}

// OperationListResult - A list of REST API operations supported by an Azure Resource Provider. It contains an URL link to
// get the next set of results.
type OperationListResult struct {
	// READ-ONLY; URL to get the next set of operation list results (if there are any).
	NextLink *string

	// READ-ONLY; List of operations supported by the resource provider
	Value []*Operation
}

// OutboundEnvironmentEndpoint - Egress endpoints which dedicated hsm service connects to for common purpose.
type OutboundEnvironmentEndpoint struct {
	// The category of endpoints accessed by the dedicated hsm service, e.g. azure-resource-management, apiserver, etc.
	Category *string

	// The endpoints that dedicated hsm service connects to
	Endpoints []*EndpointDependency
}

// OutboundEnvironmentEndpointCollection - Collection of OutboundEnvironmentEndpoint
type OutboundEnvironmentEndpointCollection struct {
	// REQUIRED; Collection of resources.
	Value []*OutboundEnvironmentEndpoint

	// READ-ONLY; Link to next page of resources.
	NextLink *string
}

// PrivateEndpoint - The private endpoint resource.
type PrivateEndpoint struct {
	// READ-ONLY; The ARM identifier for private endpoint.
	ID *string
}

// PrivateEndpointConnection - The private endpoint connection resource.
type PrivateEndpointConnection struct {
	// Modified whenever there is a change in the state of private endpoint connection.
	Etag *string

	// Resource properties.
	Properties *PrivateEndpointConnectionProperties

	// READ-ONLY; Fully qualified resource ID for the resource. E.g. "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}"
	ID *string

	// READ-ONLY; The name of the resource
	Name *string

	// READ-ONLY; Azure Resource Manager metadata containing createdBy and modifiedBy information.
	SystemData *SystemData

	// READ-ONLY; The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or "Microsoft.Storage/storageAccounts"
	Type *string
}

// PrivateEndpointConnectionListResult - List of private endpoint connections associated with the specified resource.
type PrivateEndpointConnectionListResult struct {
	// The URL to get the next set of private endpoint connections.
	NextLink *string

	// Array of private endpoint connections.
	Value []*PrivateEndpointConnection
}

// PrivateEndpointConnectionProperties - Properties of the private endpoint connection.
type PrivateEndpointConnectionProperties struct {
	// REQUIRED; A collection of information about the state of the connection between service consumer and provider.
	PrivateLinkServiceConnectionState *PrivateLinkServiceConnectionState

	// The private endpoint resource.
	PrivateEndpoint *PrivateEndpoint

	// READ-ONLY; The group ids for the private endpoint resource.
	GroupIDs []*string

	// READ-ONLY; The provisioning state of the private endpoint connection resource.
	ProvisioningState *PrivateEndpointConnectionProvisioningState
}

// PrivateLinkResource - A private link resource.
type PrivateLinkResource struct {
	// Resource properties.
	Properties *PrivateLinkResourceProperties

	// READ-ONLY; Fully qualified resource ID for the resource. E.g. "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}"
	ID *string

	// READ-ONLY; The name of the resource
	Name *string

	// READ-ONLY; Azure Resource Manager metadata containing createdBy and modifiedBy information.
	SystemData *SystemData

	// READ-ONLY; The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or "Microsoft.Storage/storageAccounts"
	Type *string
}

// PrivateLinkResourceListResult - A list of private link resources.
type PrivateLinkResourceListResult struct {
	// Array of private link resources
	Value []*PrivateLinkResource

	// READ-ONLY; URL to get the next set of operation list results (if there are any).
	NextLink *string
}

// PrivateLinkResourceProperties - Properties of a private link resource.
type PrivateLinkResourceProperties struct {
	// The private link resource private link DNS zone name.
	RequiredZoneNames []*string

	// READ-ONLY; The private link resource group id.
	GroupID *string

	// READ-ONLY; The private link resource required member names.
	RequiredMembers []*string
}

// PrivateLinkServiceConnectionState - A collection of information about the state of the connection between service consumer
// and provider.
type PrivateLinkServiceConnectionState struct {
	// A message indicating if changes on the service provider require any updates on the consumer.
	ActionsRequired *string

	// The reason for approval/rejection of the connection.
	Description *string

	// Indicates whether the connection has been Approved/Rejected/Removed by the owner of the service.
	Status *PrivateEndpointServiceConnectionStatus
}

// ProxyResource - The resource model definition for a Azure Resource Manager proxy resource. It will not have tags and a
// location
type ProxyResource struct {
	// READ-ONLY; Fully qualified resource ID for the resource. E.g. "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}"
	ID *string

	// READ-ONLY; The name of the resource
	Name *string

	// READ-ONLY; Azure Resource Manager metadata containing createdBy and modifiedBy information.
	SystemData *SystemData

	// READ-ONLY; The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or "Microsoft.Storage/storageAccounts"
	Type *string
}

// Resource - Common fields that are returned in the response for all Azure Resource Manager resources
type Resource struct {
	// READ-ONLY; Fully qualified resource ID for the resource. E.g. "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}"
	ID *string

	// READ-ONLY; The name of the resource
	Name *string

	// READ-ONLY; Azure Resource Manager metadata containing createdBy and modifiedBy information.
	SystemData *SystemData

	// READ-ONLY; The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or "Microsoft.Storage/storageAccounts"
	Type *string
}

// RestoreRequestProperties - Cloud Hsm Cluster restore information
type RestoreRequestProperties struct {
	// REQUIRED; The Azure blob storage container Uri which contains the backup
	AzureStorageBlobContainerURI *string

	// REQUIRED; An autogenerated unique string ID for labeling the backup. It contains both a UUID and a date timestamp.
	BackupID *string

	// The SAS token pointing to an Azure blob storage container. This property is reserved for Azure Backup Service.
	Token *string
}

// RestoreResult - Restore operation properties
type RestoreResult struct {
	// Backup and Restore operation common properties
	Properties *BackupRestoreBaseResultProperties
}

// SKU of the dedicated HSM
type SKU struct {
	// SKU of the dedicated HSM
	Name *SKUName
}

// SystemData - Metadata pertaining to creation and last modification of the resource.
type SystemData struct {
	// The timestamp of resource creation (UTC).
	CreatedAt *time.Time

	// The identity that created the resource.
	CreatedBy *string

	// The type of identity that created the resource.
	CreatedByType *CreatedByType

	// The timestamp of resource last modification (UTC)
	LastModifiedAt *time.Time

	// The identity that last modified the resource.
	LastModifiedBy *string

	// The type of identity that last modified the resource.
	LastModifiedByType *CreatedByType
}

// TrackedResource - The resource model definition for an Azure Resource Manager tracked top level resource which has 'tags'
// and a 'location'
type TrackedResource struct {
	// REQUIRED; The geo-location where the resource lives
	Location *string

	// Resource tags.
	Tags map[string]*string

	// READ-ONLY; Fully qualified resource ID for the resource. E.g. "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}"
	ID *string

	// READ-ONLY; The name of the resource
	Name *string

	// READ-ONLY; Azure Resource Manager metadata containing createdBy and modifiedBy information.
	SystemData *SystemData

	// READ-ONLY; The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or "Microsoft.Storage/storageAccounts"
	Type *string
}

// UserAssignedIdentity - User assigned identity properties
type UserAssignedIdentity struct {
	// READ-ONLY; The client ID of the assigned identity.
	ClientID *string

	// READ-ONLY; The principal ID of the assigned identity.
	PrincipalID *string
}
