//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package fake

import (
	"errors"
	"fmt"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"net/http"
	"strings"
	"sync"
)

// ServerFactory is a fake server for instances of the armnetworkanalytics.ClientFactory type.
type ServerFactory struct {
	DataProductsCatalogsServer DataProductsCatalogsServer
	DataProductsServer         DataProductsServer
	DataTypesServer            DataTypesServer
	OperationsServer           OperationsServer
}

// NewServerFactoryTransport creates a new instance of ServerFactoryTransport with the provided implementation.
// The returned ServerFactoryTransport instance is connected to an instance of armnetworkanalytics.ClientFactory via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewServerFactoryTransport(srv *ServerFactory) *ServerFactoryTransport {
	return &ServerFactoryTransport{
		srv: srv,
	}
}

// ServerFactoryTransport connects instances of armnetworkanalytics.ClientFactory to instances of ServerFactory.
// Don't use this type directly, use NewServerFactoryTransport instead.
type ServerFactoryTransport struct {
	srv                          *ServerFactory
	trMu                         sync.Mutex
	trDataProductsCatalogsServer *DataProductsCatalogsServerTransport
	trDataProductsServer         *DataProductsServerTransport
	trDataTypesServer            *DataTypesServerTransport
	trOperationsServer           *OperationsServerTransport
}

// Do implements the policy.Transporter interface for ServerFactoryTransport.
func (s *ServerFactoryTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	client := method[:strings.Index(method, ".")]
	var resp *http.Response
	var err error

	switch client {
	case "DataProductsCatalogsClient":
		initServer(s, &s.trDataProductsCatalogsServer, func() *DataProductsCatalogsServerTransport {
			return NewDataProductsCatalogsServerTransport(&s.srv.DataProductsCatalogsServer)
		})
		resp, err = s.trDataProductsCatalogsServer.Do(req)
	case "DataProductsClient":
		initServer(s, &s.trDataProductsServer, func() *DataProductsServerTransport { return NewDataProductsServerTransport(&s.srv.DataProductsServer) })
		resp, err = s.trDataProductsServer.Do(req)
	case "DataTypesClient":
		initServer(s, &s.trDataTypesServer, func() *DataTypesServerTransport { return NewDataTypesServerTransport(&s.srv.DataTypesServer) })
		resp, err = s.trDataTypesServer.Do(req)
	case "OperationsClient":
		initServer(s, &s.trOperationsServer, func() *OperationsServerTransport { return NewOperationsServerTransport(&s.srv.OperationsServer) })
		resp, err = s.trOperationsServer.Do(req)
	default:
		err = fmt.Errorf("unhandled client %s", client)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func initServer[T any](s *ServerFactoryTransport, dst **T, src func() *T) {
	s.trMu.Lock()
	if *dst == nil {
		*dst = src()
	}
	s.trMu.Unlock()
}
