//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package fake

import (
	"context"
	"errors"
	"fmt"
	azfake "github.com/Azure/azure-sdk-for-go/sdk/azcore/fake"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/fake/server"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/to"
	"github.com/Azure/azure-sdk-for-go/sdk/resourcemanager/oracledatabase/armoracledatabase"
	"net/http"
	"net/url"
	"regexp"
)

// AutonomousDatabaseCharacterSetsServer is a fake server for instances of the armoracledatabase.AutonomousDatabaseCharacterSetsClient type.
type AutonomousDatabaseCharacterSetsServer struct {
	// Get is the fake for method AutonomousDatabaseCharacterSetsClient.Get
	// HTTP status codes to indicate success: http.StatusOK
	Get func(ctx context.Context, location string, adbscharsetname string, options *armoracledatabase.AutonomousDatabaseCharacterSetsClientGetOptions) (resp azfake.Responder[armoracledatabase.AutonomousDatabaseCharacterSetsClientGetResponse], errResp azfake.ErrorResponder)

	// NewListByLocationPager is the fake for method AutonomousDatabaseCharacterSetsClient.NewListByLocationPager
	// HTTP status codes to indicate success: http.StatusOK
	NewListByLocationPager func(location string, options *armoracledatabase.AutonomousDatabaseCharacterSetsClientListByLocationOptions) (resp azfake.PagerResponder[armoracledatabase.AutonomousDatabaseCharacterSetsClientListByLocationResponse])
}

// NewAutonomousDatabaseCharacterSetsServerTransport creates a new instance of AutonomousDatabaseCharacterSetsServerTransport with the provided implementation.
// The returned AutonomousDatabaseCharacterSetsServerTransport instance is connected to an instance of armoracledatabase.AutonomousDatabaseCharacterSetsClient via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewAutonomousDatabaseCharacterSetsServerTransport(srv *AutonomousDatabaseCharacterSetsServer) *AutonomousDatabaseCharacterSetsServerTransport {
	return &AutonomousDatabaseCharacterSetsServerTransport{
		srv:                    srv,
		newListByLocationPager: newTracker[azfake.PagerResponder[armoracledatabase.AutonomousDatabaseCharacterSetsClientListByLocationResponse]](),
	}
}

// AutonomousDatabaseCharacterSetsServerTransport connects instances of armoracledatabase.AutonomousDatabaseCharacterSetsClient to instances of AutonomousDatabaseCharacterSetsServer.
// Don't use this type directly, use NewAutonomousDatabaseCharacterSetsServerTransport instead.
type AutonomousDatabaseCharacterSetsServerTransport struct {
	srv                    *AutonomousDatabaseCharacterSetsServer
	newListByLocationPager *tracker[azfake.PagerResponder[armoracledatabase.AutonomousDatabaseCharacterSetsClientListByLocationResponse]]
}

// Do implements the policy.Transporter interface for AutonomousDatabaseCharacterSetsServerTransport.
func (a *AutonomousDatabaseCharacterSetsServerTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	var resp *http.Response
	var err error

	switch method {
	case "AutonomousDatabaseCharacterSetsClient.Get":
		resp, err = a.dispatchGet(req)
	case "AutonomousDatabaseCharacterSetsClient.NewListByLocationPager":
		resp, err = a.dispatchNewListByLocationPager(req)
	default:
		err = fmt.Errorf("unhandled API %s", method)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func (a *AutonomousDatabaseCharacterSetsServerTransport) dispatchGet(req *http.Request) (*http.Response, error) {
	if a.srv.Get == nil {
		return nil, &nonRetriableError{errors.New("fake for method Get not implemented")}
	}
	const regexStr = `/subscriptions/(?P<subscriptionId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Oracle\.Database/locations/(?P<location>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/autonomousDatabaseCharacterSets/(?P<adbscharsetname>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)`
	regex := regexp.MustCompile(regexStr)
	matches := regex.FindStringSubmatch(req.URL.EscapedPath())
	if matches == nil || len(matches) < 3 {
		return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
	}
	locationParam, err := url.PathUnescape(matches[regex.SubexpIndex("location")])
	if err != nil {
		return nil, err
	}
	adbscharsetnameParam, err := url.PathUnescape(matches[regex.SubexpIndex("adbscharsetname")])
	if err != nil {
		return nil, err
	}
	respr, errRespr := a.srv.Get(req.Context(), locationParam, adbscharsetnameParam, nil)
	if respErr := server.GetError(errRespr, req); respErr != nil {
		return nil, respErr
	}
	respContent := server.GetResponseContent(respr)
	if !contains([]int{http.StatusOK}, respContent.HTTPStatus) {
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", respContent.HTTPStatus)}
	}
	resp, err := server.MarshalResponseAsJSON(respContent, server.GetResponse(respr).AutonomousDatabaseCharacterSet, req)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func (a *AutonomousDatabaseCharacterSetsServerTransport) dispatchNewListByLocationPager(req *http.Request) (*http.Response, error) {
	if a.srv.NewListByLocationPager == nil {
		return nil, &nonRetriableError{errors.New("fake for method NewListByLocationPager not implemented")}
	}
	newListByLocationPager := a.newListByLocationPager.get(req)
	if newListByLocationPager == nil {
		const regexStr = `/subscriptions/(?P<subscriptionId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Oracle\.Database/locations/(?P<location>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/autonomousDatabaseCharacterSets`
		regex := regexp.MustCompile(regexStr)
		matches := regex.FindStringSubmatch(req.URL.EscapedPath())
		if matches == nil || len(matches) < 2 {
			return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
		}
		locationParam, err := url.PathUnescape(matches[regex.SubexpIndex("location")])
		if err != nil {
			return nil, err
		}
		resp := a.srv.NewListByLocationPager(locationParam, nil)
		newListByLocationPager = &resp
		a.newListByLocationPager.add(req, newListByLocationPager)
		server.PagerResponderInjectNextLinks(newListByLocationPager, req, func(page *armoracledatabase.AutonomousDatabaseCharacterSetsClientListByLocationResponse, createLink func() string) {
			page.NextLink = to.Ptr(createLink())
		})
	}
	resp, err := server.PagerResponderNext(newListByLocationPager, req)
	if err != nil {
		return nil, err
	}
	if !contains([]int{http.StatusOK}, resp.StatusCode) {
		a.newListByLocationPager.remove(req)
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", resp.StatusCode)}
	}
	if !server.PagerResponderMore(newListByLocationPager) {
		a.newListByLocationPager.remove(req)
	}
	return resp, nil
}
