//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.
// DO NOT EDIT.

package armoracledatabase_test

import (
	"context"
	"log"

	"github.com/Azure/azure-sdk-for-go/sdk/azcore/to"
	"github.com/Azure/azure-sdk-for-go/sdk/azidentity"
	"github.com/Azure/azure-sdk-for-go/sdk/resourcemanager/oracledatabase/armoracledatabase"
)

// Generated from example definition: https://github.com/Azure/azure-rest-api-specs/blob/1c63635d66ae38cff18045ab416a6572d3e15f6e/specification/oracle/resource-manager/Oracle.Database/stable/2023-09-01/examples/oracleSubscriptions_listBySubscription.json
func ExampleOracleSubscriptionsClient_NewListBySubscriptionPager() {
	cred, err := azidentity.NewDefaultAzureCredential(nil)
	if err != nil {
		log.Fatalf("failed to obtain a credential: %v", err)
	}
	ctx := context.Background()
	clientFactory, err := armoracledatabase.NewClientFactory("<subscription-id>", cred, nil)
	if err != nil {
		log.Fatalf("failed to create client: %v", err)
	}
	pager := clientFactory.NewOracleSubscriptionsClient().NewListBySubscriptionPager(nil)
	for pager.More() {
		page, err := pager.NextPage(ctx)
		if err != nil {
			log.Fatalf("failed to advance page: %v", err)
		}
		for _, v := range page.Value {
			// You could use page here. We use blank identifier for just demo purposes.
			_ = v
		}
		// If the HTTP response code is 200 as defined in example definition, your page structure would look as follows. Please pay attention that all the values in the output are fake values for just demo purposes.
		// page.OracleSubscriptionListResult = armoracledatabase.OracleSubscriptionListResult{
		// 	Value: []*armoracledatabase.OracleSubscription{
		// 		{
		// 			Type: to.Ptr("Oracle.Database/oracleSubscriptions"),
		// 			ID: to.Ptr("/subscriptions/00000000-0000-0000-0000-000000000000/providers/Oracle.Database/oracleSubscriptions/default"),
		// 			Plan: &armoracledatabase.Plan{
		// 				Name: to.Ptr("plan1"),
		// 				Product: to.Ptr("product1"),
		// 				PromotionCode: to.Ptr("none"),
		// 				Publisher: to.Ptr("publisher1"),
		// 				Version: to.Ptr("alpha"),
		// 			},
		// 			Properties: &armoracledatabase.OracleSubscriptionProperties{
		// 				CloudAccountID: to.Ptr("ocid1..aaaaaa"),
		// 				CloudAccountState: to.Ptr(armoracledatabase.CloudAccountProvisioningStatePending),
		// 				ProvisioningState: to.Ptr(armoracledatabase.OracleSubscriptionProvisioningStateSucceeded),
		// 				SaasSubscriptionID: to.Ptr("saas1"),
		// 			},
		// 	}},
		// }
	}
}

// Generated from example definition: https://github.com/Azure/azure-rest-api-specs/blob/1c63635d66ae38cff18045ab416a6572d3e15f6e/specification/oracle/resource-manager/Oracle.Database/stable/2023-09-01/examples/oracleSubscriptions_get.json
func ExampleOracleSubscriptionsClient_Get() {
	cred, err := azidentity.NewDefaultAzureCredential(nil)
	if err != nil {
		log.Fatalf("failed to obtain a credential: %v", err)
	}
	ctx := context.Background()
	clientFactory, err := armoracledatabase.NewClientFactory("<subscription-id>", cred, nil)
	if err != nil {
		log.Fatalf("failed to create client: %v", err)
	}
	res, err := clientFactory.NewOracleSubscriptionsClient().Get(ctx, nil)
	if err != nil {
		log.Fatalf("failed to finish the request: %v", err)
	}
	// You could use response here. We use blank identifier for just demo purposes.
	_ = res
	// If the HTTP response code is 200 as defined in example definition, your response structure would look as follows. Please pay attention that all the values in the output are fake values for just demo purposes.
	// res.OracleSubscription = armoracledatabase.OracleSubscription{
	// 	Type: to.Ptr("Oracle.Database/oracleSubscriptions"),
	// 	ID: to.Ptr("/subscriptions/00000000-0000-0000-0000-000000000000/providers/Oracle.Database/oracleSubscriptions/default"),
	// 	Plan: &armoracledatabase.Plan{
	// 		Name: to.Ptr("plan1"),
	// 		Product: to.Ptr("product1"),
	// 		PromotionCode: to.Ptr("none"),
	// 		Publisher: to.Ptr("publisher1"),
	// 		Version: to.Ptr("alpha"),
	// 	},
	// 	Properties: &armoracledatabase.OracleSubscriptionProperties{
	// 		CloudAccountID: to.Ptr("ocid1..aaaaaa"),
	// 		CloudAccountState: to.Ptr(armoracledatabase.CloudAccountProvisioningStatePending),
	// 		ProvisioningState: to.Ptr(armoracledatabase.OracleSubscriptionProvisioningStateSucceeded),
	// 		SaasSubscriptionID: to.Ptr("saas1"),
	// 	},
	// }
}

// Generated from example definition: https://github.com/Azure/azure-rest-api-specs/blob/1c63635d66ae38cff18045ab416a6572d3e15f6e/specification/oracle/resource-manager/Oracle.Database/stable/2023-09-01/examples/oracleSubscriptions_create.json
func ExampleOracleSubscriptionsClient_BeginCreateOrUpdate() {
	cred, err := azidentity.NewDefaultAzureCredential(nil)
	if err != nil {
		log.Fatalf("failed to obtain a credential: %v", err)
	}
	ctx := context.Background()
	clientFactory, err := armoracledatabase.NewClientFactory("<subscription-id>", cred, nil)
	if err != nil {
		log.Fatalf("failed to create client: %v", err)
	}
	poller, err := clientFactory.NewOracleSubscriptionsClient().BeginCreateOrUpdate(ctx, armoracledatabase.OracleSubscription{
		Plan: &armoracledatabase.Plan{
			Name:          to.Ptr("plan1"),
			Product:       to.Ptr("product1"),
			PromotionCode: to.Ptr("none"),
			Publisher:     to.Ptr("publisher1"),
			Version:       to.Ptr("alpha"),
		},
		Properties: &armoracledatabase.OracleSubscriptionProperties{},
	}, nil)
	if err != nil {
		log.Fatalf("failed to finish the request: %v", err)
	}
	res, err := poller.PollUntilDone(ctx, nil)
	if err != nil {
		log.Fatalf("failed to pull the result: %v", err)
	}
	// You could use response here. We use blank identifier for just demo purposes.
	_ = res
	// If the HTTP response code is 200 as defined in example definition, your response structure would look as follows. Please pay attention that all the values in the output are fake values for just demo purposes.
	// res.OracleSubscription = armoracledatabase.OracleSubscription{
	// 	Type: to.Ptr("Oracle.Database/oracleSubscriptions"),
	// 	ID: to.Ptr("/subscriptions/00000000-0000-0000-0000-000000000000/providers/Oracle.Database/oracleSubscriptions/default"),
	// 	Plan: &armoracledatabase.Plan{
	// 		Name: to.Ptr("plan1"),
	// 		Product: to.Ptr("product1"),
	// 		PromotionCode: to.Ptr("none"),
	// 		Publisher: to.Ptr("publisher1"),
	// 		Version: to.Ptr("alpha"),
	// 	},
	// 	Properties: &armoracledatabase.OracleSubscriptionProperties{
	// 		CloudAccountID: to.Ptr("ocid1..aaaaaa"),
	// 		CloudAccountState: to.Ptr(armoracledatabase.CloudAccountProvisioningStatePending),
	// 		ProvisioningState: to.Ptr(armoracledatabase.OracleSubscriptionProvisioningStateSucceeded),
	// 		SaasSubscriptionID: to.Ptr("saas1"),
	// 	},
	// }
}

// Generated from example definition: https://github.com/Azure/azure-rest-api-specs/blob/1c63635d66ae38cff18045ab416a6572d3e15f6e/specification/oracle/resource-manager/Oracle.Database/stable/2023-09-01/examples/oracleSubscriptions_patch.json
func ExampleOracleSubscriptionsClient_BeginUpdate() {
	cred, err := azidentity.NewDefaultAzureCredential(nil)
	if err != nil {
		log.Fatalf("failed to obtain a credential: %v", err)
	}
	ctx := context.Background()
	clientFactory, err := armoracledatabase.NewClientFactory("<subscription-id>", cred, nil)
	if err != nil {
		log.Fatalf("failed to create client: %v", err)
	}
	poller, err := clientFactory.NewOracleSubscriptionsClient().BeginUpdate(ctx, armoracledatabase.OracleSubscriptionUpdate{}, nil)
	if err != nil {
		log.Fatalf("failed to finish the request: %v", err)
	}
	res, err := poller.PollUntilDone(ctx, nil)
	if err != nil {
		log.Fatalf("failed to pull the result: %v", err)
	}
	// You could use response here. We use blank identifier for just demo purposes.
	_ = res
	// If the HTTP response code is 200 as defined in example definition, your response structure would look as follows. Please pay attention that all the values in the output are fake values for just demo purposes.
	// res.OracleSubscription = armoracledatabase.OracleSubscription{
	// 	Type: to.Ptr("Oracle.Database/oracleSubscriptions"),
	// 	ID: to.Ptr("/subscriptions/00000000-0000-0000-0000-000000000000/providers/Oracle.Database/oracleSubscriptions/default"),
	// 	Plan: &armoracledatabase.Plan{
	// 		Name: to.Ptr("plan1"),
	// 		Product: to.Ptr("product1"),
	// 		PromotionCode: to.Ptr("none"),
	// 		Publisher: to.Ptr("publisher1"),
	// 		Version: to.Ptr("alpha"),
	// 	},
	// 	Properties: &armoracledatabase.OracleSubscriptionProperties{
	// 		CloudAccountID: to.Ptr("ocid1..aaaaaa"),
	// 		CloudAccountState: to.Ptr(armoracledatabase.CloudAccountProvisioningStatePending),
	// 		ProvisioningState: to.Ptr(armoracledatabase.OracleSubscriptionProvisioningStateSucceeded),
	// 		SaasSubscriptionID: to.Ptr("saas1"),
	// 	},
	// }
}

// Generated from example definition: https://github.com/Azure/azure-rest-api-specs/blob/1c63635d66ae38cff18045ab416a6572d3e15f6e/specification/oracle/resource-manager/Oracle.Database/stable/2023-09-01/examples/oracleSubscriptions_delete.json
func ExampleOracleSubscriptionsClient_BeginDelete() {
	cred, err := azidentity.NewDefaultAzureCredential(nil)
	if err != nil {
		log.Fatalf("failed to obtain a credential: %v", err)
	}
	ctx := context.Background()
	clientFactory, err := armoracledatabase.NewClientFactory("<subscription-id>", cred, nil)
	if err != nil {
		log.Fatalf("failed to create client: %v", err)
	}
	poller, err := clientFactory.NewOracleSubscriptionsClient().BeginDelete(ctx, nil)
	if err != nil {
		log.Fatalf("failed to finish the request: %v", err)
	}
	_, err = poller.PollUntilDone(ctx, nil)
	if err != nil {
		log.Fatalf("failed to pull the result: %v", err)
	}
}

// Generated from example definition: https://github.com/Azure/azure-rest-api-specs/blob/1c63635d66ae38cff18045ab416a6572d3e15f6e/specification/oracle/resource-manager/Oracle.Database/stable/2023-09-01/examples/oracleSubscriptions_listActivationLinks.json
func ExampleOracleSubscriptionsClient_BeginListActivationLinks() {
	cred, err := azidentity.NewDefaultAzureCredential(nil)
	if err != nil {
		log.Fatalf("failed to obtain a credential: %v", err)
	}
	ctx := context.Background()
	clientFactory, err := armoracledatabase.NewClientFactory("<subscription-id>", cred, nil)
	if err != nil {
		log.Fatalf("failed to create client: %v", err)
	}
	poller, err := clientFactory.NewOracleSubscriptionsClient().BeginListActivationLinks(ctx, nil)
	if err != nil {
		log.Fatalf("failed to finish the request: %v", err)
	}
	res, err := poller.PollUntilDone(ctx, nil)
	if err != nil {
		log.Fatalf("failed to pull the result: %v", err)
	}
	// You could use response here. We use blank identifier for just demo purposes.
	_ = res
	// If the HTTP response code is 200 as defined in example definition, your response structure would look as follows. Please pay attention that all the values in the output are fake values for just demo purposes.
	// res.ActivationLinks = armoracledatabase.ActivationLinks{
	// 	ExistingCloudAccountActivationLink: to.Ptr("https://existing-tenancy-activation-link"),
	// 	NewCloudAccountActivationLink: to.Ptr("https://new-tenancy-activation-link"),
	// }
}

// Generated from example definition: https://github.com/Azure/azure-rest-api-specs/blob/1c63635d66ae38cff18045ab416a6572d3e15f6e/specification/oracle/resource-manager/Oracle.Database/stable/2023-09-01/examples/oracleSubscriptions_listCloudAccountDetails.json
func ExampleOracleSubscriptionsClient_BeginListCloudAccountDetails() {
	cred, err := azidentity.NewDefaultAzureCredential(nil)
	if err != nil {
		log.Fatalf("failed to obtain a credential: %v", err)
	}
	ctx := context.Background()
	clientFactory, err := armoracledatabase.NewClientFactory("<subscription-id>", cred, nil)
	if err != nil {
		log.Fatalf("failed to create client: %v", err)
	}
	poller, err := clientFactory.NewOracleSubscriptionsClient().BeginListCloudAccountDetails(ctx, nil)
	if err != nil {
		log.Fatalf("failed to finish the request: %v", err)
	}
	res, err := poller.PollUntilDone(ctx, nil)
	if err != nil {
		log.Fatalf("failed to pull the result: %v", err)
	}
	// You could use response here. We use blank identifier for just demo purposes.
	_ = res
	// If the HTTP response code is 200 as defined in example definition, your response structure would look as follows. Please pay attention that all the values in the output are fake values for just demo purposes.
	// res.CloudAccountDetails = armoracledatabase.CloudAccountDetails{
	// 	CloudAccountHomeRegion: to.Ptr("East US"),
	// 	CloudAccountName: to.Ptr("Cloud Account"),
	// }
}

// Generated from example definition: https://github.com/Azure/azure-rest-api-specs/blob/1c63635d66ae38cff18045ab416a6572d3e15f6e/specification/oracle/resource-manager/Oracle.Database/stable/2023-09-01/examples/oracleSubscriptions_listSaasSubscriptionDetails.json
func ExampleOracleSubscriptionsClient_BeginListSaasSubscriptionDetails() {
	cred, err := azidentity.NewDefaultAzureCredential(nil)
	if err != nil {
		log.Fatalf("failed to obtain a credential: %v", err)
	}
	ctx := context.Background()
	clientFactory, err := armoracledatabase.NewClientFactory("<subscription-id>", cred, nil)
	if err != nil {
		log.Fatalf("failed to create client: %v", err)
	}
	poller, err := clientFactory.NewOracleSubscriptionsClient().BeginListSaasSubscriptionDetails(ctx, nil)
	if err != nil {
		log.Fatalf("failed to finish the request: %v", err)
	}
	res, err := poller.PollUntilDone(ctx, nil)
	if err != nil {
		log.Fatalf("failed to pull the result: %v", err)
	}
	// You could use response here. We use blank identifier for just demo purposes.
	_ = res
	// If the HTTP response code is 200 as defined in example definition, your response structure would look as follows. Please pay attention that all the values in the output are fake values for just demo purposes.
	// res.SaasSubscriptionDetails = armoracledatabase.SaasSubscriptionDetails{
	// 	ID: to.Ptr("1234567890"),
	// 	IsAutoRenew: to.Ptr(true),
	// 	IsFreeTrial: to.Ptr(false),
	// 	OfferID: to.Ptr("offer1"),
	// 	PlanID: to.Ptr("silver"),
	// 	PublisherID: to.Ptr("Oracle"),
	// 	PurchaserEmailID: to.Ptr("test@test.com"),
	// 	PurchaserTenantID: to.Ptr("1234567890"),
	// 	SaasSubscriptionStatus: to.Ptr("PendingFulfillmentStart"),
	// 	SubscriptionName: to.Ptr("Oracle"),
	// 	TermUnit: to.Ptr("P1M"),
	// 	TimeCreated: to.Ptr(func() time.Time { t, _ := time.Parse(time.RFC3339Nano, "2022-03-07T00:00:00.000Z"); return t}()),
	// }
}
