//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package fake

import (
	"errors"
	"fmt"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"net/http"
	"strings"
	"sync"
)

// ServerFactory is a fake server for instances of the armorbital.ClientFactory type.
type ServerFactory struct {
	AvailableGroundStationsServer AvailableGroundStationsServer
	ContactProfilesServer         ContactProfilesServer
	ContactsServer                ContactsServer
	OperationsServer              OperationsServer
	OperationsResultsServer       OperationsResultsServer
	SpacecraftsServer             SpacecraftsServer
}

// NewServerFactoryTransport creates a new instance of ServerFactoryTransport with the provided implementation.
// The returned ServerFactoryTransport instance is connected to an instance of armorbital.ClientFactory via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewServerFactoryTransport(srv *ServerFactory) *ServerFactoryTransport {
	return &ServerFactoryTransport{
		srv: srv,
	}
}

// ServerFactoryTransport connects instances of armorbital.ClientFactory to instances of ServerFactory.
// Don't use this type directly, use NewServerFactoryTransport instead.
type ServerFactoryTransport struct {
	srv                             *ServerFactory
	trMu                            sync.Mutex
	trAvailableGroundStationsServer *AvailableGroundStationsServerTransport
	trContactProfilesServer         *ContactProfilesServerTransport
	trContactsServer                *ContactsServerTransport
	trOperationsServer              *OperationsServerTransport
	trOperationsResultsServer       *OperationsResultsServerTransport
	trSpacecraftsServer             *SpacecraftsServerTransport
}

// Do implements the policy.Transporter interface for ServerFactoryTransport.
func (s *ServerFactoryTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	client := method[:strings.Index(method, ".")]
	var resp *http.Response
	var err error

	switch client {
	case "AvailableGroundStationsClient":
		initServer(s, &s.trAvailableGroundStationsServer, func() *AvailableGroundStationsServerTransport {
			return NewAvailableGroundStationsServerTransport(&s.srv.AvailableGroundStationsServer)
		})
		resp, err = s.trAvailableGroundStationsServer.Do(req)
	case "ContactProfilesClient":
		initServer(s, &s.trContactProfilesServer, func() *ContactProfilesServerTransport {
			return NewContactProfilesServerTransport(&s.srv.ContactProfilesServer)
		})
		resp, err = s.trContactProfilesServer.Do(req)
	case "ContactsClient":
		initServer(s, &s.trContactsServer, func() *ContactsServerTransport { return NewContactsServerTransport(&s.srv.ContactsServer) })
		resp, err = s.trContactsServer.Do(req)
	case "OperationsClient":
		initServer(s, &s.trOperationsServer, func() *OperationsServerTransport { return NewOperationsServerTransport(&s.srv.OperationsServer) })
		resp, err = s.trOperationsServer.Do(req)
	case "OperationsResultsClient":
		initServer(s, &s.trOperationsResultsServer, func() *OperationsResultsServerTransport {
			return NewOperationsResultsServerTransport(&s.srv.OperationsResultsServer)
		})
		resp, err = s.trOperationsResultsServer.Do(req)
	case "SpacecraftsClient":
		initServer(s, &s.trSpacecraftsServer, func() *SpacecraftsServerTransport { return NewSpacecraftsServerTransport(&s.srv.SpacecraftsServer) })
		resp, err = s.trSpacecraftsServer.Do(req)
	default:
		err = fmt.Errorf("unhandled client %s", client)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func initServer[T any](s *ServerFactoryTransport, dst **T, src func() *T) {
	s.trMu.Lock()
	if *dst == nil {
		*dst = src()
	}
	s.trMu.Unlock()
}
