//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package fake

import (
	"context"
	"errors"
	"fmt"
	azfake "github.com/Azure/azure-sdk-for-go/sdk/azcore/fake"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/fake/server"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"github.com/Azure/azure-sdk-for-go/sdk/resourcemanager/powerplatform/armpowerplatform"
	"net/http"
	"net/url"
	"regexp"
)

// PrivateLinkResourcesServer is a fake server for instances of the armpowerplatform.PrivateLinkResourcesClient type.
type PrivateLinkResourcesServer struct {
	// Get is the fake for method PrivateLinkResourcesClient.Get
	// HTTP status codes to indicate success: http.StatusOK
	Get func(ctx context.Context, resourceGroupName string, enterprisePolicyName string, groupName string, options *armpowerplatform.PrivateLinkResourcesClientGetOptions) (resp azfake.Responder[armpowerplatform.PrivateLinkResourcesClientGetResponse], errResp azfake.ErrorResponder)

	// NewListByEnterprisePolicyPager is the fake for method PrivateLinkResourcesClient.NewListByEnterprisePolicyPager
	// HTTP status codes to indicate success: http.StatusOK
	NewListByEnterprisePolicyPager func(resourceGroupName string, enterprisePolicyName string, options *armpowerplatform.PrivateLinkResourcesClientListByEnterprisePolicyOptions) (resp azfake.PagerResponder[armpowerplatform.PrivateLinkResourcesClientListByEnterprisePolicyResponse])
}

// NewPrivateLinkResourcesServerTransport creates a new instance of PrivateLinkResourcesServerTransport with the provided implementation.
// The returned PrivateLinkResourcesServerTransport instance is connected to an instance of armpowerplatform.PrivateLinkResourcesClient via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewPrivateLinkResourcesServerTransport(srv *PrivateLinkResourcesServer) *PrivateLinkResourcesServerTransport {
	return &PrivateLinkResourcesServerTransport{
		srv:                            srv,
		newListByEnterprisePolicyPager: newTracker[azfake.PagerResponder[armpowerplatform.PrivateLinkResourcesClientListByEnterprisePolicyResponse]](),
	}
}

// PrivateLinkResourcesServerTransport connects instances of armpowerplatform.PrivateLinkResourcesClient to instances of PrivateLinkResourcesServer.
// Don't use this type directly, use NewPrivateLinkResourcesServerTransport instead.
type PrivateLinkResourcesServerTransport struct {
	srv                            *PrivateLinkResourcesServer
	newListByEnterprisePolicyPager *tracker[azfake.PagerResponder[armpowerplatform.PrivateLinkResourcesClientListByEnterprisePolicyResponse]]
}

// Do implements the policy.Transporter interface for PrivateLinkResourcesServerTransport.
func (p *PrivateLinkResourcesServerTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	var resp *http.Response
	var err error

	switch method {
	case "PrivateLinkResourcesClient.Get":
		resp, err = p.dispatchGet(req)
	case "PrivateLinkResourcesClient.NewListByEnterprisePolicyPager":
		resp, err = p.dispatchNewListByEnterprisePolicyPager(req)
	default:
		err = fmt.Errorf("unhandled API %s", method)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func (p *PrivateLinkResourcesServerTransport) dispatchGet(req *http.Request) (*http.Response, error) {
	if p.srv.Get == nil {
		return nil, &nonRetriableError{errors.New("fake for method Get not implemented")}
	}
	const regexStr = `/subscriptions/(?P<subscriptionId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/resourceGroups/(?P<resourceGroupName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Microsoft\.PowerPlatform/enterprisePolicies/(?P<enterprisePolicyName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/privateLinkResources/(?P<groupName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)`
	regex := regexp.MustCompile(regexStr)
	matches := regex.FindStringSubmatch(req.URL.EscapedPath())
	if matches == nil || len(matches) < 4 {
		return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
	}
	resourceGroupNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("resourceGroupName")])
	if err != nil {
		return nil, err
	}
	enterprisePolicyNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("enterprisePolicyName")])
	if err != nil {
		return nil, err
	}
	groupNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("groupName")])
	if err != nil {
		return nil, err
	}
	respr, errRespr := p.srv.Get(req.Context(), resourceGroupNameParam, enterprisePolicyNameParam, groupNameParam, nil)
	if respErr := server.GetError(errRespr, req); respErr != nil {
		return nil, respErr
	}
	respContent := server.GetResponseContent(respr)
	if !contains([]int{http.StatusOK}, respContent.HTTPStatus) {
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", respContent.HTTPStatus)}
	}
	resp, err := server.MarshalResponseAsJSON(respContent, server.GetResponse(respr).PrivateLinkResource, req)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func (p *PrivateLinkResourcesServerTransport) dispatchNewListByEnterprisePolicyPager(req *http.Request) (*http.Response, error) {
	if p.srv.NewListByEnterprisePolicyPager == nil {
		return nil, &nonRetriableError{errors.New("fake for method NewListByEnterprisePolicyPager not implemented")}
	}
	newListByEnterprisePolicyPager := p.newListByEnterprisePolicyPager.get(req)
	if newListByEnterprisePolicyPager == nil {
		const regexStr = `/subscriptions/(?P<subscriptionId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/resourceGroups/(?P<resourceGroupName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Microsoft\.PowerPlatform/enterprisePolicies/(?P<enterprisePolicyName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/privateLinkResources`
		regex := regexp.MustCompile(regexStr)
		matches := regex.FindStringSubmatch(req.URL.EscapedPath())
		if matches == nil || len(matches) < 3 {
			return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
		}
		resourceGroupNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("resourceGroupName")])
		if err != nil {
			return nil, err
		}
		enterprisePolicyNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("enterprisePolicyName")])
		if err != nil {
			return nil, err
		}
		resp := p.srv.NewListByEnterprisePolicyPager(resourceGroupNameParam, enterprisePolicyNameParam, nil)
		newListByEnterprisePolicyPager = &resp
		p.newListByEnterprisePolicyPager.add(req, newListByEnterprisePolicyPager)
	}
	resp, err := server.PagerResponderNext(newListByEnterprisePolicyPager, req)
	if err != nil {
		return nil, err
	}
	if !contains([]int{http.StatusOK}, resp.StatusCode) {
		p.newListByEnterprisePolicyPager.remove(req)
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", resp.StatusCode)}
	}
	if !server.PagerResponderMore(newListByEnterprisePolicyPager) {
		p.newListByEnterprisePolicyPager.remove(req)
	}
	return resp, nil
}
