//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package fake

import (
	"context"
	"errors"
	"fmt"
	azfake "github.com/Azure/azure-sdk-for-go/sdk/azcore/fake"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/fake/server"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"github.com/Azure/azure-sdk-for-go/sdk/resourcemanager/security/armsecurity"
	"net/http"
	"net/url"
	"regexp"
)

// IotSecuritySolutionAnalyticsServer is a fake server for instances of the armsecurity.IotSecuritySolutionAnalyticsClient type.
type IotSecuritySolutionAnalyticsServer struct {
	// Get is the fake for method IotSecuritySolutionAnalyticsClient.Get
	// HTTP status codes to indicate success: http.StatusOK
	Get func(ctx context.Context, resourceGroupName string, solutionName string, options *armsecurity.IotSecuritySolutionAnalyticsClientGetOptions) (resp azfake.Responder[armsecurity.IotSecuritySolutionAnalyticsClientGetResponse], errResp azfake.ErrorResponder)

	// List is the fake for method IotSecuritySolutionAnalyticsClient.List
	// HTTP status codes to indicate success: http.StatusOK
	List func(ctx context.Context, resourceGroupName string, solutionName string, options *armsecurity.IotSecuritySolutionAnalyticsClientListOptions) (resp azfake.Responder[armsecurity.IotSecuritySolutionAnalyticsClientListResponse], errResp azfake.ErrorResponder)
}

// NewIotSecuritySolutionAnalyticsServerTransport creates a new instance of IotSecuritySolutionAnalyticsServerTransport with the provided implementation.
// The returned IotSecuritySolutionAnalyticsServerTransport instance is connected to an instance of armsecurity.IotSecuritySolutionAnalyticsClient via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewIotSecuritySolutionAnalyticsServerTransport(srv *IotSecuritySolutionAnalyticsServer) *IotSecuritySolutionAnalyticsServerTransport {
	return &IotSecuritySolutionAnalyticsServerTransport{srv: srv}
}

// IotSecuritySolutionAnalyticsServerTransport connects instances of armsecurity.IotSecuritySolutionAnalyticsClient to instances of IotSecuritySolutionAnalyticsServer.
// Don't use this type directly, use NewIotSecuritySolutionAnalyticsServerTransport instead.
type IotSecuritySolutionAnalyticsServerTransport struct {
	srv *IotSecuritySolutionAnalyticsServer
}

// Do implements the policy.Transporter interface for IotSecuritySolutionAnalyticsServerTransport.
func (i *IotSecuritySolutionAnalyticsServerTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	var resp *http.Response
	var err error

	switch method {
	case "IotSecuritySolutionAnalyticsClient.Get":
		resp, err = i.dispatchGet(req)
	case "IotSecuritySolutionAnalyticsClient.List":
		resp, err = i.dispatchList(req)
	default:
		err = fmt.Errorf("unhandled API %s", method)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func (i *IotSecuritySolutionAnalyticsServerTransport) dispatchGet(req *http.Request) (*http.Response, error) {
	if i.srv.Get == nil {
		return nil, &nonRetriableError{errors.New("fake for method Get not implemented")}
	}
	const regexStr = `/subscriptions/(?P<subscriptionId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/resourceGroups/(?P<resourceGroupName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Microsoft\.Security/iotSecuritySolutions/(?P<solutionName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/analyticsModels/default`
	regex := regexp.MustCompile(regexStr)
	matches := regex.FindStringSubmatch(req.URL.EscapedPath())
	if matches == nil || len(matches) < 3 {
		return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
	}
	resourceGroupNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("resourceGroupName")])
	if err != nil {
		return nil, err
	}
	solutionNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("solutionName")])
	if err != nil {
		return nil, err
	}
	respr, errRespr := i.srv.Get(req.Context(), resourceGroupNameParam, solutionNameParam, nil)
	if respErr := server.GetError(errRespr, req); respErr != nil {
		return nil, respErr
	}
	respContent := server.GetResponseContent(respr)
	if !contains([]int{http.StatusOK}, respContent.HTTPStatus) {
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", respContent.HTTPStatus)}
	}
	resp, err := server.MarshalResponseAsJSON(respContent, server.GetResponse(respr).IoTSecuritySolutionAnalyticsModel, req)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func (i *IotSecuritySolutionAnalyticsServerTransport) dispatchList(req *http.Request) (*http.Response, error) {
	if i.srv.List == nil {
		return nil, &nonRetriableError{errors.New("fake for method List not implemented")}
	}
	const regexStr = `/subscriptions/(?P<subscriptionId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/resourceGroups/(?P<resourceGroupName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Microsoft\.Security/iotSecuritySolutions/(?P<solutionName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/analyticsModels`
	regex := regexp.MustCompile(regexStr)
	matches := regex.FindStringSubmatch(req.URL.EscapedPath())
	if matches == nil || len(matches) < 3 {
		return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
	}
	resourceGroupNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("resourceGroupName")])
	if err != nil {
		return nil, err
	}
	solutionNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("solutionName")])
	if err != nil {
		return nil, err
	}
	respr, errRespr := i.srv.List(req.Context(), resourceGroupNameParam, solutionNameParam, nil)
	if respErr := server.GetError(errRespr, req); respErr != nil {
		return nil, respErr
	}
	respContent := server.GetResponseContent(respr)
	if !contains([]int{http.StatusOK}, respContent.HTTPStatus) {
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", respContent.HTTPStatus)}
	}
	resp, err := server.MarshalResponseAsJSON(respContent, server.GetResponse(respr).IoTSecuritySolutionAnalyticsModelList, req)
	if err != nil {
		return nil, err
	}
	return resp, nil
}
