//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package fake

import (
	"context"
	"errors"
	"fmt"
	azfake "github.com/Azure/azure-sdk-for-go/sdk/azcore/fake"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/fake/server"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"github.com/Azure/azure-sdk-for-go/sdk/resourcemanager/securityinsights/armsecurityinsights/v2"
	"net/http"
	"net/url"
	"regexp"
)

// DomainWhoisServer is a fake server for instances of the armsecurityinsights.DomainWhoisClient type.
type DomainWhoisServer struct {
	// Get is the fake for method DomainWhoisClient.Get
	// HTTP status codes to indicate success: http.StatusOK
	Get func(ctx context.Context, resourceGroupName string, domain string, options *armsecurityinsights.DomainWhoisClientGetOptions) (resp azfake.Responder[armsecurityinsights.DomainWhoisClientGetResponse], errResp azfake.ErrorResponder)
}

// NewDomainWhoisServerTransport creates a new instance of DomainWhoisServerTransport with the provided implementation.
// The returned DomainWhoisServerTransport instance is connected to an instance of armsecurityinsights.DomainWhoisClient via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewDomainWhoisServerTransport(srv *DomainWhoisServer) *DomainWhoisServerTransport {
	return &DomainWhoisServerTransport{srv: srv}
}

// DomainWhoisServerTransport connects instances of armsecurityinsights.DomainWhoisClient to instances of DomainWhoisServer.
// Don't use this type directly, use NewDomainWhoisServerTransport instead.
type DomainWhoisServerTransport struct {
	srv *DomainWhoisServer
}

// Do implements the policy.Transporter interface for DomainWhoisServerTransport.
func (d *DomainWhoisServerTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	var resp *http.Response
	var err error

	switch method {
	case "DomainWhoisClient.Get":
		resp, err = d.dispatchGet(req)
	default:
		err = fmt.Errorf("unhandled API %s", method)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func (d *DomainWhoisServerTransport) dispatchGet(req *http.Request) (*http.Response, error) {
	if d.srv.Get == nil {
		return nil, &nonRetriableError{errors.New("fake for method Get not implemented")}
	}
	const regexStr = `/subscriptions/(?P<subscriptionId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/resourceGroups/(?P<resourceGroupName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Microsoft\.SecurityInsights/enrichment/domain/whois/`
	regex := regexp.MustCompile(regexStr)
	matches := regex.FindStringSubmatch(req.URL.EscapedPath())
	if matches == nil || len(matches) < 2 {
		return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
	}
	qp := req.URL.Query()
	resourceGroupNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("resourceGroupName")])
	if err != nil {
		return nil, err
	}
	domainParam, err := url.QueryUnescape(qp.Get("domain"))
	if err != nil {
		return nil, err
	}
	respr, errRespr := d.srv.Get(req.Context(), resourceGroupNameParam, domainParam, nil)
	if respErr := server.GetError(errRespr, req); respErr != nil {
		return nil, respErr
	}
	respContent := server.GetResponseContent(respr)
	if !contains([]int{http.StatusOK}, respContent.HTTPStatus) {
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", respContent.HTTPStatus)}
	}
	resp, err := server.MarshalResponseAsJSON(respContent, server.GetResponse(respr).EnrichmentDomainWhois, req)
	if err != nil {
		return nil, err
	}
	return resp, nil
}
