//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package fake

import (
	"context"
	"errors"
	"fmt"
	azfake "github.com/Azure/azure-sdk-for-go/sdk/azcore/fake"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/fake/server"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"github.com/Azure/azure-sdk-for-go/sdk/resourcemanager/securityinsights/armsecurityinsights/v2"
	"net/http"
	"net/url"
	"regexp"
)

// IPGeodataServer is a fake server for instances of the armsecurityinsights.IPGeodataClient type.
type IPGeodataServer struct {
	// Get is the fake for method IPGeodataClient.Get
	// HTTP status codes to indicate success: http.StatusOK
	Get func(ctx context.Context, resourceGroupName string, ipAddress string, options *armsecurityinsights.IPGeodataClientGetOptions) (resp azfake.Responder[armsecurityinsights.IPGeodataClientGetResponse], errResp azfake.ErrorResponder)
}

// NewIPGeodataServerTransport creates a new instance of IPGeodataServerTransport with the provided implementation.
// The returned IPGeodataServerTransport instance is connected to an instance of armsecurityinsights.IPGeodataClient via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewIPGeodataServerTransport(srv *IPGeodataServer) *IPGeodataServerTransport {
	return &IPGeodataServerTransport{srv: srv}
}

// IPGeodataServerTransport connects instances of armsecurityinsights.IPGeodataClient to instances of IPGeodataServer.
// Don't use this type directly, use NewIPGeodataServerTransport instead.
type IPGeodataServerTransport struct {
	srv *IPGeodataServer
}

// Do implements the policy.Transporter interface for IPGeodataServerTransport.
func (i *IPGeodataServerTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	var resp *http.Response
	var err error

	switch method {
	case "IPGeodataClient.Get":
		resp, err = i.dispatchGet(req)
	default:
		err = fmt.Errorf("unhandled API %s", method)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func (i *IPGeodataServerTransport) dispatchGet(req *http.Request) (*http.Response, error) {
	if i.srv.Get == nil {
		return nil, &nonRetriableError{errors.New("fake for method Get not implemented")}
	}
	const regexStr = `/subscriptions/(?P<subscriptionId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/resourceGroups/(?P<resourceGroupName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Microsoft\.SecurityInsights/enrichment/ip/geodata/`
	regex := regexp.MustCompile(regexStr)
	matches := regex.FindStringSubmatch(req.URL.EscapedPath())
	if matches == nil || len(matches) < 2 {
		return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
	}
	qp := req.URL.Query()
	resourceGroupNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("resourceGroupName")])
	if err != nil {
		return nil, err
	}
	ipAddressParam, err := url.QueryUnescape(qp.Get("ipAddress"))
	if err != nil {
		return nil, err
	}
	respr, errRespr := i.srv.Get(req.Context(), resourceGroupNameParam, ipAddressParam, nil)
	if respErr := server.GetError(errRespr, req); respErr != nil {
		return nil, respErr
	}
	respContent := server.GetResponseContent(respr)
	if !contains([]int{http.StatusOK}, respContent.HTTPStatus) {
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", respContent.HTTPStatus)}
	}
	resp, err := server.MarshalResponseAsJSON(respContent, server.GetResponse(respr).EnrichmentIPGeodata, req)
	if err != nil {
		return nil, err
	}
	return resp, nil
}
