//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package fake

import (
	"errors"
	"fmt"
	azfake "github.com/Azure/azure-sdk-for-go/sdk/azcore/fake"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/fake/server"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/to"
	"github.com/Azure/azure-sdk-for-go/sdk/resourcemanager/securityinsights/armsecurityinsights/v2"
	"net/http"
	"net/url"
	"regexp"
)

// SourceControlServer is a fake server for instances of the armsecurityinsights.SourceControlClient type.
type SourceControlServer struct {
	// NewListRepositoriesPager is the fake for method SourceControlClient.NewListRepositoriesPager
	// HTTP status codes to indicate success: http.StatusOK
	NewListRepositoriesPager func(resourceGroupName string, workspaceName string, repoType armsecurityinsights.RepoType, options *armsecurityinsights.SourceControlClientListRepositoriesOptions) (resp azfake.PagerResponder[armsecurityinsights.SourceControlClientListRepositoriesResponse])
}

// NewSourceControlServerTransport creates a new instance of SourceControlServerTransport with the provided implementation.
// The returned SourceControlServerTransport instance is connected to an instance of armsecurityinsights.SourceControlClient via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewSourceControlServerTransport(srv *SourceControlServer) *SourceControlServerTransport {
	return &SourceControlServerTransport{
		srv:                      srv,
		newListRepositoriesPager: newTracker[azfake.PagerResponder[armsecurityinsights.SourceControlClientListRepositoriesResponse]](),
	}
}

// SourceControlServerTransport connects instances of armsecurityinsights.SourceControlClient to instances of SourceControlServer.
// Don't use this type directly, use NewSourceControlServerTransport instead.
type SourceControlServerTransport struct {
	srv                      *SourceControlServer
	newListRepositoriesPager *tracker[azfake.PagerResponder[armsecurityinsights.SourceControlClientListRepositoriesResponse]]
}

// Do implements the policy.Transporter interface for SourceControlServerTransport.
func (s *SourceControlServerTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	var resp *http.Response
	var err error

	switch method {
	case "SourceControlClient.NewListRepositoriesPager":
		resp, err = s.dispatchNewListRepositoriesPager(req)
	default:
		err = fmt.Errorf("unhandled API %s", method)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func (s *SourceControlServerTransport) dispatchNewListRepositoriesPager(req *http.Request) (*http.Response, error) {
	if s.srv.NewListRepositoriesPager == nil {
		return nil, &nonRetriableError{errors.New("fake for method NewListRepositoriesPager not implemented")}
	}
	newListRepositoriesPager := s.newListRepositoriesPager.get(req)
	if newListRepositoriesPager == nil {
		const regexStr = `/subscriptions/(?P<subscriptionId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/resourceGroups/(?P<resourceGroupName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Microsoft\.OperationalInsights/workspaces/(?P<workspaceName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Microsoft\.SecurityInsights/listRepositories`
		regex := regexp.MustCompile(regexStr)
		matches := regex.FindStringSubmatch(req.URL.EscapedPath())
		if matches == nil || len(matches) < 3 {
			return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
		}
		body, err := server.UnmarshalRequestAsJSON[armsecurityinsights.RepoType](req)
		if err != nil {
			return nil, err
		}
		resourceGroupNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("resourceGroupName")])
		if err != nil {
			return nil, err
		}
		workspaceNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("workspaceName")])
		if err != nil {
			return nil, err
		}
		resp := s.srv.NewListRepositoriesPager(resourceGroupNameParam, workspaceNameParam, body, nil)
		newListRepositoriesPager = &resp
		s.newListRepositoriesPager.add(req, newListRepositoriesPager)
		server.PagerResponderInjectNextLinks(newListRepositoriesPager, req, func(page *armsecurityinsights.SourceControlClientListRepositoriesResponse, createLink func() string) {
			page.NextLink = to.Ptr(createLink())
		})
	}
	resp, err := server.PagerResponderNext(newListRepositoriesPager, req)
	if err != nil {
		return nil, err
	}
	if !contains([]int{http.StatusOK}, resp.StatusCode) {
		s.newListRepositoriesPager.remove(req)
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", resp.StatusCode)}
	}
	if !server.PagerResponderMore(newListRepositoriesPager) {
		s.newListRepositoriesPager.remove(req)
	}
	return resp, nil
}
