//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package fake

import (
	"errors"
	"fmt"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"net/http"
	"strings"
	"sync"
)

// ServerFactory is a fake server for instances of the armserialconsole.ClientFactory type.
type ServerFactory struct {
	MicrosoftSerialConsoleServer MicrosoftSerialConsoleServer
	SerialPortsServer            SerialPortsServer
}

// NewServerFactoryTransport creates a new instance of ServerFactoryTransport with the provided implementation.
// The returned ServerFactoryTransport instance is connected to an instance of armserialconsole.ClientFactory via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewServerFactoryTransport(srv *ServerFactory) *ServerFactoryTransport {
	return &ServerFactoryTransport{
		srv: srv,
	}
}

// ServerFactoryTransport connects instances of armserialconsole.ClientFactory to instances of ServerFactory.
// Don't use this type directly, use NewServerFactoryTransport instead.
type ServerFactoryTransport struct {
	srv                            *ServerFactory
	trMu                           sync.Mutex
	trMicrosoftSerialConsoleServer *MicrosoftSerialConsoleServerTransport
	trSerialPortsServer            *SerialPortsServerTransport
}

// Do implements the policy.Transporter interface for ServerFactoryTransport.
func (s *ServerFactoryTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	client := method[:strings.Index(method, ".")]
	var resp *http.Response
	var err error

	switch client {
	case "MicrosoftSerialConsoleClient":
		initServer(s, &s.trMicrosoftSerialConsoleServer, func() *MicrosoftSerialConsoleServerTransport {
			return NewMicrosoftSerialConsoleServerTransport(&s.srv.MicrosoftSerialConsoleServer)
		})
		resp, err = s.trMicrosoftSerialConsoleServer.Do(req)
	case "SerialPortsClient":
		initServer(s, &s.trSerialPortsServer, func() *SerialPortsServerTransport { return NewSerialPortsServerTransport(&s.srv.SerialPortsServer) })
		resp, err = s.trSerialPortsServer.Do(req)
	default:
		err = fmt.Errorf("unhandled client %s", client)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func initServer[T any](s *ServerFactoryTransport, dst **T, src func() *T) {
	s.trMu.Lock()
	if *dst == nil {
		*dst = src()
	}
	s.trMu.Unlock()
}
