//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package fake

import (
	"errors"
	"fmt"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"net/http"
	"strings"
	"sync"
)

// ServerFactory is a fake server for instances of the armwebpubsub.ClientFactory type.
type ServerFactory struct {
	Server                                  Server
	CustomCertificatesServer                CustomCertificatesServer
	CustomDomainsServer                     CustomDomainsServer
	HubsServer                              HubsServer
	OperationsServer                        OperationsServer
	PrivateEndpointConnectionsServer        PrivateEndpointConnectionsServer
	PrivateLinkResourcesServer              PrivateLinkResourcesServer
	ReplicaSharedPrivateLinkResourcesServer ReplicaSharedPrivateLinkResourcesServer
	ReplicasServer                          ReplicasServer
	SharedPrivateLinkResourcesServer        SharedPrivateLinkResourcesServer
	UsagesServer                            UsagesServer
}

// NewServerFactoryTransport creates a new instance of ServerFactoryTransport with the provided implementation.
// The returned ServerFactoryTransport instance is connected to an instance of armwebpubsub.ClientFactory via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewServerFactoryTransport(srv *ServerFactory) *ServerFactoryTransport {
	return &ServerFactoryTransport{
		srv: srv,
	}
}

// ServerFactoryTransport connects instances of armwebpubsub.ClientFactory to instances of ServerFactory.
// Don't use this type directly, use NewServerFactoryTransport instead.
type ServerFactoryTransport struct {
	srv                                       *ServerFactory
	trMu                                      sync.Mutex
	trServer                                  *ServerTransport
	trCustomCertificatesServer                *CustomCertificatesServerTransport
	trCustomDomainsServer                     *CustomDomainsServerTransport
	trHubsServer                              *HubsServerTransport
	trOperationsServer                        *OperationsServerTransport
	trPrivateEndpointConnectionsServer        *PrivateEndpointConnectionsServerTransport
	trPrivateLinkResourcesServer              *PrivateLinkResourcesServerTransport
	trReplicaSharedPrivateLinkResourcesServer *ReplicaSharedPrivateLinkResourcesServerTransport
	trReplicasServer                          *ReplicasServerTransport
	trSharedPrivateLinkResourcesServer        *SharedPrivateLinkResourcesServerTransport
	trUsagesServer                            *UsagesServerTransport
}

// Do implements the policy.Transporter interface for ServerFactoryTransport.
func (s *ServerFactoryTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	client := method[:strings.Index(method, ".")]
	var resp *http.Response
	var err error

	switch client {
	case "Client":
		initServer(s, &s.trServer, func() *ServerTransport { return NewServerTransport(&s.srv.Server) })
		resp, err = s.trServer.Do(req)
	case "CustomCertificatesClient":
		initServer(s, &s.trCustomCertificatesServer, func() *CustomCertificatesServerTransport {
			return NewCustomCertificatesServerTransport(&s.srv.CustomCertificatesServer)
		})
		resp, err = s.trCustomCertificatesServer.Do(req)
	case "CustomDomainsClient":
		initServer(s, &s.trCustomDomainsServer, func() *CustomDomainsServerTransport {
			return NewCustomDomainsServerTransport(&s.srv.CustomDomainsServer)
		})
		resp, err = s.trCustomDomainsServer.Do(req)
	case "HubsClient":
		initServer(s, &s.trHubsServer, func() *HubsServerTransport { return NewHubsServerTransport(&s.srv.HubsServer) })
		resp, err = s.trHubsServer.Do(req)
	case "OperationsClient":
		initServer(s, &s.trOperationsServer, func() *OperationsServerTransport { return NewOperationsServerTransport(&s.srv.OperationsServer) })
		resp, err = s.trOperationsServer.Do(req)
	case "PrivateEndpointConnectionsClient":
		initServer(s, &s.trPrivateEndpointConnectionsServer, func() *PrivateEndpointConnectionsServerTransport {
			return NewPrivateEndpointConnectionsServerTransport(&s.srv.PrivateEndpointConnectionsServer)
		})
		resp, err = s.trPrivateEndpointConnectionsServer.Do(req)
	case "PrivateLinkResourcesClient":
		initServer(s, &s.trPrivateLinkResourcesServer, func() *PrivateLinkResourcesServerTransport {
			return NewPrivateLinkResourcesServerTransport(&s.srv.PrivateLinkResourcesServer)
		})
		resp, err = s.trPrivateLinkResourcesServer.Do(req)
	case "ReplicaSharedPrivateLinkResourcesClient":
		initServer(s, &s.trReplicaSharedPrivateLinkResourcesServer, func() *ReplicaSharedPrivateLinkResourcesServerTransport {
			return NewReplicaSharedPrivateLinkResourcesServerTransport(&s.srv.ReplicaSharedPrivateLinkResourcesServer)
		})
		resp, err = s.trReplicaSharedPrivateLinkResourcesServer.Do(req)
	case "ReplicasClient":
		initServer(s, &s.trReplicasServer, func() *ReplicasServerTransport { return NewReplicasServerTransport(&s.srv.ReplicasServer) })
		resp, err = s.trReplicasServer.Do(req)
	case "SharedPrivateLinkResourcesClient":
		initServer(s, &s.trSharedPrivateLinkResourcesServer, func() *SharedPrivateLinkResourcesServerTransport {
			return NewSharedPrivateLinkResourcesServerTransport(&s.srv.SharedPrivateLinkResourcesServer)
		})
		resp, err = s.trSharedPrivateLinkResourcesServer.Do(req)
	case "UsagesClient":
		initServer(s, &s.trUsagesServer, func() *UsagesServerTransport { return NewUsagesServerTransport(&s.srv.UsagesServer) })
		resp, err = s.trUsagesServer.Do(req)
	default:
		err = fmt.Errorf("unhandled client %s", client)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func initServer[T any](s *ServerFactoryTransport, dst **T, src func() *T) {
	s.trMu.Lock()
	if *dst == nil {
		*dst = src()
	}
	s.trMu.Unlock()
}
